unit tvUserConfig;

interface

uses
  Windows, ShellAPI, SysUtils, Classes, Generics.Collections, utFunctions, VirtualTrees, Graphics;

const
{$IFDEF DEBUG}
  CONFIG_FILE_NAME = 'TweetVoice.debug.cfg';
{$ELSE}
  CONFIG_FILE_NAME = 'TweetVoice.cfg';
{$ENDIF}
  TWITTER_BASE_URL: string = 'http://twitter.com/';
  API_TWITTER_BASE_URL: string = 'http://api.twitter.com/';
  SEARCH_TWITTER_BASE_URL: string = 'http://search.twitter.com/';

type
  TCaptionType = (ctNone, ctBalloon, ctCaptionNormal, ctCaptionWindow);
  TURLShortenerSite = (usBitLy, usTinyURL, usIsGd);
  TTabType = (ttHomeTimeline, ttUserTimeline, ttReplies, ttDirectMessages, ttFavorites, ttSearch, ttList,
    ttRetweetedByMe, ttRetweetsOfMe, ttSentDirectMessages);
  TUsersTabType = (utFollowingMe, utFollowedByMe, utList);
  TStatusViewType = (svNormal, svJustFit, svSingleLine);

  TCaptionFont = class(TPersistent)
  private
    FSize: Integer;
    FIsBold: Boolean;
    FIsItalic: Boolean;
    FBorderColor: TColor;
    FName: string;
    FBorderSize: Integer;
    FColor: TColor;
  protected
    procedure AssignTo(ADest: TPersistent); override;
  public
    constructor Create;
    destructor Destroy; override;
    function CreateFont: TFont;
  published
    property Size: Integer read FSize write FSize;
    property IsBold: Boolean read FIsBold write FIsBold;
    property IsItalic: Boolean read FIsItalic write FIsItalic;
    property BorderSize: Integer read FBorderSize write FBorderSize;
    property BorderColor: TColor read FBorderColor write FBorderColor;
    property Name: string read FName write FName;
    property Color: TColor read FColor write FColor;
  end;

  TUserAuth = class(TCollectionItem)
  private
    FUserName: AnsiString;
    FOAuthToken: AnsiString;
    FOAuthTokenSecret: AnsiString;
  public
    procedure CryptPassword;
  published
    property UserName: AnsiString read FUserName write FUserName;
    property OAuthToken: AnsiString read FOAuthToken write FOAuthToken;
    property OAuthTokenSecret: AnsiString read FOAuthTokenSecret write FOAuthTokenSecret;
  end;

  TWindowRect = class(TPersistent)
  private
    FLeft: Integer;
    FTop: Integer;
    FWidth: Integer;
    FHeight: Integer;
  published
    property Left: Integer read FLeft write FLeft;
    property Top: Integer read FTop write FTop;
    property Width: Integer read FWidth write FWidth;
    property Height: Integer read FHeight write FHeight;
  end;

  TProxyConfig = class(TPersistent)
  private
    FUseProxy: Boolean;
    FPort: Integer;
    FHost: string;
  published
    property UseProxy: Boolean read FUseProxy write FUseProxy;
    property Host: string read FHost write FHost;
    property Port: Integer read FPort write FPort default 8080;

    constructor Create;
  end;

  TFontConfig = class(TPersistent)
  private
    FFontName: string;
    FFontSize: Integer;
  published
    property FontName: string read FFontName write FFontName;
    property FontSize: Integer read FFontSize write FFontSize;

    constructor Create;
  end;

  TURLShortenerConfig = class(TPersistent)
  private
    FURLShortenerSite: TURLShortenerSite;
    FUserName: AnsiString;
    FPassword: AnsiString;
    FAPIKey: AnsiString;
  published
    property UserName: AnsiString read FUserName write FUserName;
    property Password: AnsiString read FPassword write FPassword;
    property APIKey: AnsiString read FAPIKey write FAPIKey;
    property URLShortenerSite: TURLShortenerSite read FURLShortenerSite write FURLShortenerSite default usBitLy;
    constructor Create;
    procedure CryptPassword;
    function Enabled: Boolean;
  end;

  TTabConfig = class(TCollectionItem)
  private
    FTabType: TTabType;
    FIsPublicMode: Boolean;
    FSearchWord: string;
    FUserName: string;
    FListName: string;
    FIsSpeechTab: Boolean;
  published
    property TabType: TTabType read FTabType write FTabType;
    property SearchWord: string read FSearchWord write FSearchWord;
    property UserName: string read FUserName write FUserName;
    property ListName: string read FListName write FListName;
    property IsPublicMode: Boolean read FIsPublicMode write FIsPublicMode default false;
    property IsSpeechTab: Boolean read FIsSpeechTab write FIsSpeechTab default false;
  end;

  TUsersTabConfig = class(TCollectionItem)
  private
    FTabType: TUsersTabType;
    FListName: string;
  published
    property TabType: TUsersTabType read FTabType write FTabType;
    property ListName: string read FListName write FListName;
  end;

  TPlaySoundConfig = class(TPersistent)
  private
    FPlaySound: Boolean;
    FSoundFileName: string;
    FSynchPlaySound: Boolean;
  published
    constructor Create;
    property PlaySound: Boolean read FPlaySound write FPlaySound default false;
    property SoundFileName: string read FSoundFileName write FSoundFileName;
    property SynchPlaySound: Boolean read FSynchPlaySound write FSynchPlaySound default true;
  end;

  TSpeechConfig = class(TCollectionItem)
  private
    FRate: Integer;
    FVolume: Integer;
    FVoiceName: string;
    FLang: string;
    FInitialSpeech: string;
    FWaitingTimeSec: Integer;
    FPronounciations: string;
    FMuteWaitingTimeMSec: Integer;
    FRetweetSpeech: string;
  published
    property VoiceName: string read FVoiceName write FVoiceName;
    property Lang: string read FLang write FLang;
    property Rate: Integer read FRate write FRate default 0;
    property Volume: Integer read FVolume write FVolume default 100;
    property WaitingTimeSec: Integer read FWaitingTimeSec write FWaitingTimeSec default 2;
    property MuteWaitingTimeMSec: Integer read FMuteWaitingTimeMSec write FMuteWaitingTimeMSec default 150;
    property InitialSpeech: string read FInitialSpeech write FInitialSpeech;
    property RetweetSpeech: string read FRetweetSpeech write FRetweetSpeech;
    property Pronounciations: string read FPronounciations write FPronounciations;
  protected
    procedure AssignTo(ADest: TPersistent); override;
  public
    Modified: Boolean;

    procedure Initialize;

    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  end;

  TAutoUpdateSec = class(TPersistent)
  private
    FHomeTimeline: Integer;
    FUserTimeline: Integer;
    FReplies: Integer;
    FDirectMessages: Integer;
    FSentDirectMessages: Integer;
    FFavorites: Integer;
    FPublicList: Integer;
    FPrivateList: Integer;
    FSearch: Integer;
    FRetweetedByMe: Integer;
    FRetweetsOfMe: Integer;
  public
    constructor Create;
  published
    property HomeTimeline: Integer read FHomeTimeline write FHomeTimeline default 60;
    property UserTimeline: Integer read FUserTimeline write FUserTimeline default 600;
    property Replies: Integer read FReplies write FReplies default 600;
    property DirectMessages: Integer read FDirectMessages write FDirectMessages default 600;
    property SentDirectMessages: Integer read FSentDirectMessages write FSentDirectMessages default 600;
    property Favorites: Integer read FFavorites write FFavorites default 600;
    property PublicList: Integer read FPublicList write FPublicList default 60;
    property PrivateList: Integer read FPrivateList write FPrivateList default 600;
    property RetweetedByMe: Integer read FRetweetedByMe write FRetweetedByMe default 600;
    property RetweetsOfMe: Integer read FRetweetsOfMe write FRetweetsOfMe default 600;
    property Search: Integer read FSearch write FSearch default 60;
  end;

  TTreeViewColumn = class(TCollectionItem)
  private
    FText: string;
    FWidth: Integer;
    FPosition: Integer;
  published
    property Text: string read FText write FText;
    property Width: Integer read FWidth write FWidth;
    property Position: Integer read FPosition write FPosition;
  end;

  TUserConfig = class(TPersistent)
  public
    constructor Create;
    destructor Destroy; override;
  private
    FDefaultUserName: string;
    FAutoUpdateSec: TAutoUpdateSec;
    FFormMainRect: TWindowRect;
    FFormUsersRect: TWindowRect;
    FFormCaptionRect: TWindowRect;
    FUserOAuths: TCollection;
    FProxyConfig: TProxyConfig;
    FSpeechConfigs: TCollection;
    FListViewFont: TFontConfig;
    FRelativeDateString: Boolean;
    FRequestStatusesCount: Integer;
    FPauseSpeech: Boolean;
    FAutoRefresh: Boolean;
    FShowFormLoginStartUp: Boolean;
    FAutoRequestOlderStatuses: Boolean;
    FFormMainListViewColumns: TCollection;
    FCaptionType: TCaptionType;
    FCaptionFont: TCaptionFont;
    FCaptionFontUserName: TCaptionFont;
    FCaptionFontDateTime: TCaptionFont;
    FMuteSpeech: Boolean;
    FURLShortenerConfig: TURLShortenerConfig;
    FComplementScreenName: Boolean;
    FTabConfigs: TCollection;
    FPlaySoundConfig: TPlaySoundConfig;
    FStatusViewType: TStatusViewType;
    FPanelPostVisible: Boolean;
    procedure CryptUserAuth;
    function GetCaptionFont: TCaptionFont;
  protected
    FIsLoaded: Boolean;
  published
    property RequestStatusesCount: Integer read FRequestStatusesCount write FRequestStatusesCount default 20;
    property AutoUpdateSec: TAutoUpdateSec read FAutoUpdateSec write FAutoUpdateSec;
    property PanelPostVisible: Boolean read FPanelPostVisible write FPanelPostVisible default true;
    property DefaultUserName: string read FDefaultUserName write FDefaultUserName;
    property FormMainRect: TWindowRect read FFormMainRect write FFormMainRect;
    property FormUsersRect: TWindowRect read FFormUsersRect write FFormUsersRect;
    property FormCaptionRect: TWindowRect read FFormCaptionRect write FFormCaptionRect;
    property UserOAuths: TCollection read FUserOAuths write FUserOAuths;
    property ProxyConfig: TProxyConfig read FProxyConfig write FProxyConfig;
    property ListViewFont: TFontConfig read FListViewFont write FListViewFont;
    property RelativeDateString: Boolean read FRelativeDateString write FRelativeDateString default true;
    property PauseSpeech: Boolean read FPauseSpeech write FPauseSpeech;
    property MuteSpeech: Boolean read FMuteSpeech write FMuteSpeech;
    property AutoRefresh: Boolean read FAutoRefresh write FAutoRefresh default true;
    property AutoRequestOlderStatuses
      : Boolean read FAutoRequestOlderStatuses write FAutoRequestOlderStatuses default true;
    property StatusViewType: TStatusViewType read FStatusViewType write FStatusViewType default svNormal;
    property SpeechConfigs: TCollection read FSpeechConfigs write FSpeechConfigs;
    property ShowFormLoginStartUp: Boolean read FShowFormLoginStartUp write FShowFormLoginStartUp default true;
    property ComplementScreenName: Boolean read FComplementScreenName write FComplementScreenName default true;
    property FormMainListViewColumns: TCollection read FFormMainListViewColumns write FFormMainListViewColumns;
    property CaptionType: TCaptionType read FCaptionType write FCaptionType default ctCaptionNormal;
    property CaptionFont: TCaptionFont read GetCaptionFont write FCaptionFont;
    property CaptionFontUserName: TCaptionFont read FCaptionFontUserName write FCaptionFontUserName;
    property CaptionFontDateTime: TCaptionFont read FCaptionFontDateTime write FCaptionFontDateTime;
    property URLShortenerConfig: TURLShortenerConfig read FURLShortenerConfig write FURLShortenerConfig;
    property TabConfigs: TCollection read FTabConfigs write FTabConfigs;
    property PlaySoundConfig: TPlaySoundConfig read FPlaySoundConfig write FPlaySoundConfig;
  public
    procedure Save(FileName: String); overload;
    procedure Save; overload;

    function GetSpeechConfig(ALang: string): TSpeechConfig;
    function IsLoaded: Boolean;
    procedure ApplyFontSizeToForms;
    class function Load(FileName: String): TUserConfig;

    class procedure LoadUserConfig;
    class procedure SaveUserConfig;
  end;

  TOpenWebAsBrowserThread = class(TThread)
  public
    Url: string;
  protected
    procedure Execute; override;
  end;

var
  UserConfig: TUserConfig;
  UserConfigFilePath: string;

function OpenWebAsBrowser(AUrl: string): Boolean;
procedure GetConfigFromTreeView(AColumns: TCollection; ATreeView: TVirtualStringTree);
procedure SetConfigToTreeView(AColumns: TCollection; ATreeView: TVirtualStringTree);
function GetDateString(ADateTime: TDateTime; ALang: string): string;

implementation

uses XmlDoc, XMLIntf, OmniXML, OmniXMLPersistent, ShlObj, Forms, tvFormMain, tvFormUsers;

function OpenWebAsBrowser(AUrl: string): Boolean;
var
  openWebThread: TOpenWebAsBrowserThread;
begin
  openWebThread := TOpenWebAsBrowserThread.Create(true);
  openWebThread.Url := AUrl;
  openWebThread.FreeOnTerminate := true;
  openWebThread.Start;
  Result := true;
end;

procedure GetConfigFromTreeView(AColumns: TCollection; ATreeView: TVirtualStringTree);
var
  i: Integer;
  ACol: TTreeViewColumn;
begin
  AColumns.Clear;
  for i := 0 to ATreeView.Header.Columns.Count - 1 do
  begin
    ACol := AColumns.Add as TTreeViewColumn;
    ACol.Text := ATreeView.Header.Columns.Items[i].Text;
    ACol.Position := ATreeView.Header.Columns.Items[i].Position;
    ACol.Width := ATreeView.Header.Columns.Items[i].Width;
  end;
end;

procedure SetConfigToTreeView(AColumns: TCollection; ATreeView: TVirtualStringTree);
var
  i, j: Integer;
  ACol: TTreeViewColumn;
  Count: Integer;
begin
  Count := ATreeView.Header.Columns.Count;
  if (Count > AColumns.Count) then
  begin
    Count := AColumns.Count;
  end;
  for i := 0 to Count - 1 do
  begin
    for j := 0 to Count - 1 do
    begin
      ACol := AColumns.Items[j] as TTreeViewColumn;
      if (ACol.Position = i) then
      begin
        ATreeView.Header.Columns.Items[j].Position := ACol.Position;
        ATreeView.Header.Columns.Items[j].Width := ACol.Width;

        break;
      end;
    end;
  end;
end;

function GetDateString(ADateTime: TDateTime; ALang: string): string;
begin
  if (UserConfig.RelativeDateString) then
  begin
    Result := Utils.GetRelativeDateString(ADateTime, ALang);
  end
  else
  begin
    Result := FormatDateTime('yy/mm/dd hh:mm:ss', ADateTime);
  end;
end;

{ UserConfig }

procedure TUserConfig.ApplyFontSizeToForms;
begin
  FormMainTweetVoice.VirtualTreeView.Font.Name := ListViewFont.FontName;
  FormMainTweetVoice.VirtualTreeView.Font.Size := ListViewFont.FontSize;
  FormMainTweetVoice.VirtualTreeView.ReinitChildren(nil, true);

  FormUsers.VirtualTreeView.ReinitChildren(nil, true);
  FormUsers.VirtualTreeView.Repaint;
end;

constructor TUserConfig.Create;
begin
  FAutoUpdateSec := TAutoUpdateSec.Create;
  FUserOAuths := TCollection.Create(TUserAuth);
  FSpeechConfigs := TCollection.Create(TSpeechConfig);
  FFormMainListViewColumns := TCollection.Create(TTreeViewColumn);
  FFormMainRect := TWindowRect.Create;
  FFormUsersRect := TWindowRect.Create;
  FFormCaptionRect := TWindowRect.Create;
  FProxyConfig := TProxyConfig.Create;
  FListViewFont := TFontConfig.Create;
  FRelativeDateString := true;
  {
  FAutoUpdateSec.HomeTimeline := 60;
  FAutoUpdateSec.UserTimeline := 600;
  FAutoUpdateSec.Replies := 600;
  FAutoUpdateSec.DirectMessages := 600;
  FAutoUpdateSec.Favorites := 600;
  FAutoUpdateSec.PublicList := 60;
  FAutoUpdateSec.PrivateList := 600;
  FAutoUpdateSec.Search := 60;
  }
  FRequestStatusesCount := 20;
  FAutoRefresh := true;
  FShowFormLoginStartUp := true;
  FAutoRequestOlderStatuses := true;
  FCaptionType := ctCaptionNormal;
  FCaptionFont := TCaptionFont.Create;
  FCaptionFontUserName := TCaptionFont.Create;
  FCaptionFontUserName.Size := 14;
  FCaptionFontUserName.BorderSize := 3;
  FCaptionFontUserName.Color := RGB(128, 0, 0);
  FCaptionFontUserName.BorderColor := clWhite;

  FCaptionFontDateTime := TCaptionFont.Create;
  FCaptionFontDateTime.Size := 14;
  FCaptionFontDateTime.BorderSize := 3;
  FCaptionFontDateTime.Color := RGB(200, 255, 200);
  FCaptionFontDateTime.BorderColor := RGB(0, 128, 0);

  FComplementScreenName := true;
  FURLShortenerConfig := TURLShortenerConfig.Create;
  FTabConfigs := TCollection.Create(TTabConfig);
  FPlaySoundConfig := TPlaySoundConfig.Create;
  FStatusViewType := svNormal;
  FPanelPostVisible := true;
end;

procedure TUserConfig.CryptUserAuth;
var
  i: Integer;
  user: TUserAuth;
begin
  for i := 0 to FUserOAuths.Count - 1 do
  begin
    user := FUserOAuths.Items[i] as TUserAuth;
    user.CryptPassword;
  end;
  FURLShortenerConfig.CryptPassword;
end;

destructor TUserConfig.Destroy;
begin
  FPlaySoundConfig.Free;
  FTabConfigs.Free;
  FURLShortenerConfig.Free;
  FCaptionFontDateTime.Free;
  FCaptionFontUserName.Free;
  FCaptionFont.Free;
  FFormMainListViewColumns.Free;
  FSpeechConfigs.Free;
  FListViewFont.Free;
  FProxyConfig.Free;
  FFormCaptionRect.Free;
  FFormMainRect.Free;
  FFormUsersRect.Free;
  FUserOAuths.Free;
  FAutoUpdateSec.Free;
  inherited;
end;

function TUserConfig.GetCaptionFont: TCaptionFont;
begin
  if Not Assigned(FCaptionFont) then
  begin
    FCaptionFont := TCaptionFont.Create;
  end;
  Result := FCaptionFont;
end;

function TUserConfig.GetSpeechConfig(ALang: string): TSpeechConfig;
var
  speechConfig: TSpeechConfig;
  i: Integer;
begin

  Result := nil;
  // vbN
  for i := 0 to FSpeechConfigs.Count - 1 do
  begin
    speechConfig := FSpeechConfigs.Items[i] as TSpeechConfig;

    if (speechConfig.Lang = ALang) then
    begin
      Result := speechConfig;
    end;
  end;

  if not Assigned(Result) then
  begin
    Result := FSpeechConfigs.Items[0] as TSpeechConfig; // ԂftHg
  end;
end;

function TUserConfig.IsLoaded: Boolean;
begin
  Result := FIsLoaded;
end;

class function TUserConfig.Load(FileName: String): TUserConfig;
begin
  Result := TUserConfig.Create;
  TOmniXMLReader.LoadFromFile(Result, FileName);
end;

class procedure TUserConfig.LoadUserConfig;
var
  userConfigDir: string;
  speechConfig: TSpeechConfig;
  i: Integer;
begin
  UserConfigFilePath := Utils.GetSpecialFolder(CSIDL_APPDATA) + '\TweetVoice\' + CONFIG_FILE_NAME;

  userConfigDir := ExtractFileDir(UserConfigFilePath);
  if (Not DirectoryExists(userConfigDir)) then
  begin
    MkDir(userConfigDir);
  end;

  try
    if (FileExists(UserConfigFilePath)) then
    begin
      UserConfig := TUserConfig.Load(UserConfigFilePath);
      UserConfig.FIsLoaded := true;
      UserConfig.CryptUserAuth;
    end
    else
    begin
      UserConfig := TUserConfig.Create;
    end;
  except
    on E: Exception do
      UserConfig := TUserConfig.Create;
  end;
  if (UserConfig.SpeechConfigs.Count = 0) then
  begin
    speechConfig := UserConfig.SpeechConfigs.Add as TSpeechConfig;
    speechConfig.Initialize;
    speechConfig.VoiceName := 'VW Misaki';
    speechConfig.InitialSpeech := '{speaker}̂Ԃ₫B';
    speechConfig.RetweetSpeech := '{retweeter}{speaker}̂Ԃ₫cC[g܂B';

    speechConfig := UserConfig.SpeechConfigs.Add as TSpeechConfig;
    speechConfig.Initialize;
    speechConfig.VoiceName := 'Microsoft Anna - English (United States)';
    speechConfig.InitialSpeech := '{speaker}''s tweet. ';
    speechConfig.RetweetSpeech := '{retweeter} retweeted {speaker}''s tweet. ';

    speechConfig.Lang := 'en';
  end;

  for i := 0 to UserConfig.SpeechConfigs.Count - 1 do
  begin
    speechConfig := UserConfig.SpeechConfigs.Items[i] as TSpeechConfig;
    if (speechConfig.FRetweetSpeech = '') then
    begin
      if (speechConfig.Lang = 'ja') then
      begin
        speechConfig.RetweetSpeech := '{retweeter}{speaker}̂Ԃ₫cC[g܂B';
      end
      else
      begin
        speechConfig.RetweetSpeech := '{retweeter} retweeted {speaker}''s tweet. ';
      end;
    end;
  end;
end;

procedure TUserConfig.Save(FileName: String);
begin
  UserConfig.CryptUserAuth;
  TOmniXMLWriter.SaveToFile(Self, FileName, pfNodes, ofIndent);
  UserConfig.CryptUserAuth;
end;

procedure TUserConfig.Save;
begin
  Save(UserConfigFilePath);
end;

class procedure TUserConfig.SaveUserConfig;
begin
  UserConfig.Save(UserConfigFilePath);
  UserConfig.Free;
end;

{ TUserPass }

procedure TUserAuth.CryptPassword;
var
  i: Integer;
begin
  for i := 1 to Length(FOAuthTokenSecret) do
  begin
    FOAuthTokenSecret[i] := AnsiChar(Ord(FOAuthTokenSecret[i]) xor 15);
  end;
end;

{ TProxyConfig }

constructor TProxyConfig.Create;
begin
  FPort := 8080;
  FUseProxy := false;
end;

{ TFontConfig }

constructor TFontConfig.Create;
begin
  FFontName := '';
  FFontSize := 0;
end;

{ TOpenWebAsBrowserThread }

procedure TOpenWebAsBrowserThread.Execute;
begin
  inherited;
  ShellExecute(0, 'open', PChar(Url), nil, nil, SW_SHOW);
end;

{ TAutoUpdateSec }

constructor TAutoUpdateSec.Create;
begin
  FHomeTimeline := 60;
  FUserTimeline := 60;
  FReplies := 600;
  FDirectMessages := 600;
  FSentDirectMessages := 600;
  FFavorites := 600;
  FPublicList := 60;
  FPrivateList := 600;
  FSearch := 60;
  FRetweetedByMe := 600;
  FRetweetsOfMe := 600;
end;

{ TSpeechConfig }

procedure TSpeechConfig.AssignTo(ADest: TPersistent);
var
  destConfig: TSpeechConfig;
begin
  destConfig := ADest as TSpeechConfig;

  destConfig.FRate := FRate;
  destConfig.FLang := FLang;
  destConfig.FVolume := FVolume;
  destConfig.FVoiceName := FVoiceName;
  destConfig.FWaitingTimeSec := FWaitingTimeSec;
  destConfig.FInitialSpeech := FInitialSpeech;
  destConfig.FRetweetSpeech := FRetweetSpeech;
  destConfig.FPronounciations := FPronounciations;
  destConfig.Modified := true;
  destConfig.FMuteWaitingTimeMSec := FMuteWaitingTimeMSec;
end;

constructor TSpeechConfig.Create(ACollection: TCollection);
begin
  Inherited Create(ACollection);
end;

destructor TSpeechConfig.Destroy;
begin
  inherited;
end;

procedure TSpeechConfig.Initialize;
begin
  FLang := 'ja';
  FRate := 0;
  FVolume := 100;
  FWaitingTimeSec := 2;
  FMuteWaitingTimeMSec := 150;
end;

{ TCaptionFont }

procedure TCaptionFont.AssignTo(ADest: TPersistent);
var
  CaptionFont: TCaptionFont;
begin
  CaptionFont := ADest as TCaptionFont;
  CaptionFont.FSize := FSize;
  CaptionFont.FIsBold := FIsBold;
  CaptionFont.FIsItalic := FIsItalic;
  CaptionFont.FBorderColor := FBorderColor;
  CaptionFont.FColor := FColor;
  CaptionFont.FName := FName;
  CaptionFont.FBorderSize := FBorderSize;
end;

constructor TCaptionFont.Create;
begin
  Self.FSize := 17;
  Self.IsBold := true;
  Self.FIsItalic := false;
  Self.FName := 'Tahoma';
  Self.FColor := clWhite;
  Self.BorderColor := RGB(0, 0, 128);
  Self.FBorderSize := 4;
end;

destructor TCaptionFont.Destroy;
begin
  inherited;
end;

function TCaptionFont.CreateFont: TFont;
begin
  Result := TFont.Create;
  Result.Name := FName;
  Result.Size := FSize;
  Result.Color := FColor;
  if (IsBold) then
  begin
    Result.Style := Result.Style + [fsBold];
  end;
  if (IsItalic) then
  begin
    Result.Style := Result.Style + [fsItalic];
  end;
end;

{ TURLShortenerConfig }

constructor TURLShortenerConfig.Create;
begin
  FURLShortenerSite := usTinyURL;
end;

procedure TURLShortenerConfig.CryptPassword;
var
  i: Integer;
begin
  for i := 1 to Length(FPassword) do
  begin
    FPassword[i] := AnsiChar(Ord(FPassword[i]) xor 15);
  end;

  for i := 1 to Length(FAPIKey) do
  begin
    FAPIKey[i] := AnsiChar(Ord(FAPIKey[i]) xor 15);
  end;

end;

function TURLShortenerConfig.Enabled: Boolean;
begin
  if (FURLShortenerSite <> usTinyURL) then
  begin
    Result := (FUserName <> '') and (FAPIKey <> '');
  end
  else
  begin
    Result := true;
  end;
end;

{ TPlaySoundConfig }

constructor TPlaySoundConfig.Create;
begin
  FSynchPlaySound := true;
end;

end.
