unit tvOAuth;

interface

uses
  IdBaseComponent, IdComponent, IdTCPConnection, IdSSLOpenSSLHeaders, IdSSLOpenSSL, IdTCPClient, IdHTTP, OAuth,
  SysUtils, tvUserConfig, Types, Classes, SyncObjs, HTTPUtil, StrUtils;

const
  OAUTH_CONSUMER_KEY = '0QE4MoOnbmloBSzeTQD5qQ';
  OAUTH_CONSUMER_SECRET = 'BmkG63Sa5JWDZWK7rn1uUpbOfYVbDVvvLDtiVWbVro';
  OAUTH_REQUEST_TOKEN_URL = 'https://api.twitter.com/oauth/request_token';
  OAUTH_AUTHORIZE_URL = 'http://twitter.com/oauth/authorize';
  OAUTH_ACCESS_TOKEN_URL = 'https://api.twitter.com/oauth/access_token';

type

  TTwitterAuth = class
  private
    FHTTP: TIdHTTP;
    FConsumer: TOAuthConsumer;
    FHMACPost: TOAuthSignatureMethod_HMAC_SHA1;
    FHMACGet: TOAuthSignatureMethod_HMAC_SHA1;
    FRequest: TOAuthRequest;
    FIdSSLIOHandlerSocket: TIdSSLIOHandlerSocketOpenSSL;
    FExceptionMessage: string;
    FOAuthToken: string;
    FOAuthTokenSecret: string;
    FScreenName: string;
    FCriticalSection: TCriticalSection;
    procedure SetTokens(const AResponse: string);
    procedure SetOAuthToken(const Value: string);
    procedure SetOAuthTokenSecret(const Value: string);
  public

    constructor Create;
    destructor Destroy; override;

    function Authorize(AUserName, APassword: string): boolean; overload;
    function Authorize(APinCode: string): boolean; overload;

    function CheckAuthToken: boolean;
    property OAuthToken: string read FOAuthToken write FOAuthToken;
    property OAuthTokenSecret: string read FOAuthTokenSecret write FOAuthTokenSecret;
    property ScreenName: string read FScreenName;
    function GetTwitterAppAuthURL: string;

    function GETCommand(URL: string; AParams: TStrings): string;
    function POSTCommand(URL: string; AParams: TStrings): TStrings;
    property ExceptionMessage: string read FExceptionMessage;
    property Consumer: TOAuthConsumer read FConsumer;
  end;

var
  TwitterAuth: TTwitterAuth;

implementation

uses
  tvCommunicator;
{ TTwitterAuth }

(* xAuthFؕ̂ߔp~ *)
function TTwitterAuth.Authorize(AUserName, APassword: string): boolean;
var
  Response, URL: string;
  endpos: integer;
begin
  Result := false;
  // params := TStringList.Create;
  // Request := TOAuthRequest.Create(url);
  try
    begin
      URL := OAUTH_ACCESS_TOKEN_URL;
      FRequest := FRequest.FromConsumerAndToken(FConsumer, nil, URL);
      FRequest.HTTPURL := URL;
      FRequest.Parameters.Add('x_auth_mode=client_auth');
      FRequest.Parameters.Add('x_auth_password=' + OAuth.TOAuthUtil.urlEncodeRFC3986(APassword));
      FRequest.Parameters.Add('x_auth_username=' + OAuth.TOAuthUtil.urlEncodeRFC3986(AUserName));
      FRequest.Sign_Request(FHMACPost, FConsumer, nil);
      FHTTP.Request.Clear;
      FHTTP.HTTPOptions := [hoKeepOrigProtocol];
      try
        TCommunicator.SetProxy(FHTTP);
        Response := FHTTP.Post(URL, FRequest.Parameters);
      except
        on E: Exception do
        begin
          FExceptionMessage := E.Message;
          Exit;
        end;
      end;

      endpos := AnsiPos('&oauth_token_secret=', Response);
      FOAuthToken := Copy(Response, 13, endpos - 13);
      Response := Copy(Response, endpos, Length(Response));
      endpos := AnsiPos('&user', Response);
      FOAuthTokenSecret := '';
      FOAuthTokenSecret := Copy(Response, 21, endpos - 21);
      Result := true;
    end
  finally
    // params.Free;
    // Request.Free;
  end;

end;

function TTwitterAuth.Authorize(APinCode: string): boolean;
var
  Response, URL: string;
begin
  Result := false;

  URL := OAUTH_ACCESS_TOKEN_URL;
  FRequest := FRequest.FromConsumerAndToken(FConsumer, nil, URL);
  FRequest.HTTPURL := URL;
  FHTTP.Request.Clear;
  FRequest.Parameters.Add('oauth_verifier=' + OAuth.TOAuthUtil.urlEncodeRFC3986(APinCode));
  FRequest.Parameters.Add('oauth_token=' + OAuth.TOAuthUtil.urlEncodeRFC3986(FOAuthToken));
  // token := TOAuthToken.Create(FOAuthToken, FOAuthTokenSecret);
  try
    FHTTP.HTTPOptions := [hoKeepOrigProtocol];
    FRequest.Sign_Request(FHMACPost, FConsumer, nil);
    try
      TCommunicator.SetProxy(FHTTP);
      Response := FHTTP.Post(URL, FRequest.Parameters);
    except
      on E: Exception do
      begin
        FExceptionMessage := E.Message;
        Exit;
      end;
    end;
    SetTokens(Response);
    Result := true;
  finally
    // token.Free;
  end;
end;

function TTwitterAuth.CheckAuthToken: boolean;
var
  ss: TStrings;
  URL: string;
  res: string;
begin
  Result := false;
  ss := TStringList.Create;
  try
    ss.Add('count=1');
    URL := GETCommand(API_TWITTER_BASE_URL + 'statuses/home_timeline.xml', ss);
    FHTTP.Request.Clear;
    TCommunicator.SetProxy(FHTTP);
    try
      res := FHTTP.Get(URL);
      Result := true;
    except
      on E: Exception do
      begin
        FExceptionMessage := E.Message;
      end;
    end;
  finally
    ss.Free;
  end;
end;

constructor TTwitterAuth.Create;
begin
  FIdSSLIOHandlerSocket := TIdSSLIOHandlerSocketOpenSSL.Create(nil);
  FHTTP := TIdHTTP.Create(nil);
  FHTTP.IOHandler := FIdSSLIOHandlerSocket;
  FConsumer := TOAuthConsumer.Create(OAUTH_CONSUMER_KEY, OAUTH_CONSUMER_SECRET);
  FHMACPost := TOAuthSignatureMethod_HMAC_SHA1.Create('POST');
  FHMACGet := TOAuthSignatureMethod_HMAC_SHA1.Create('GET');
  FRequest := TOAuthRequest.Create('');
  FCriticalSection := TCriticalSection.Create;
end;

destructor TTwitterAuth.Destroy;
begin
  FCriticalSection.Free;
  FIdSSLIOHandlerSocket.Free;
  FHTTP.Free;
  FConsumer.Free;
  FHMACGet.Free;
  FHMACPost.Free;
  FRequest.Free;
  inherited;
end;

procedure TTwitterAuth.SetOAuthToken(const Value: string);
begin
  FOAuthToken := Value;
end;

procedure TTwitterAuth.SetOAuthTokenSecret(const Value: string);
begin
  FOAuthTokenSecret := Value;
end;

procedure TTwitterAuth.SetTokens(const AResponse: string);
var
  strings: TStringDynArray;
  s: string;
begin
  strings := StringToStringArray(AResponse, '&');
  for s in strings do
  begin
    if (StartsStr('oauth_token=', s)) then
    begin
      FOAuthToken := Copy(s, 13);
    end
    else if (StartsStr('oauth_token_secret=', s)) then
    begin
      FOAuthTokenSecret := Copy(s, 20);
    end
    else if (StartsStr('screen_name=', s)) then
    begin
      FScreenName := Copy(s, 13);
    end;
  end;
end;

function TTwitterAuth.GETCommand(URL: string; AParams: TStrings): string;
var
  pos: integer;
  token: TOAuthToken;
  s: string;
begin
  FCriticalSection.Acquire;
  try
    FRequest := FRequest.FromConsumerAndToken(FConsumer, nil, URL);
    FRequest.HTTPURL := URL;
    token := TOAuthToken.Create(FOAuthToken, FOAuthTokenSecret);
    try
      FRequest := FRequest.FromConsumerAndToken(FConsumer, token, URL);
      if (AParams <> nil) then
      begin
        for s in AParams do
        begin
          FRequest.Parameters.Add(s);
        end;
      end;

      FRequest.Sign_Request(FHMACGet, Consumer, token);
      pos := AnsiPos('?', URL);
      Assert(pos = 0, 'URLɈƌ듮삷\܂I');
      URL := URL + '?' + FRequest.GetString;
      Result := URL;
    finally
      token.Free;
    end;
  finally
    FCriticalSection.Leave
  end;
end;

function TTwitterAuth.GetTwitterAppAuthURL: string;
var
  Response, URL: string;
  beginPos: integer;
begin
  Result := '';
  try
    begin
      URL := OAUTH_REQUEST_TOKEN_URL;
      FRequest := FRequest.FromConsumerAndToken(FConsumer, nil, URL);
      FRequest.HTTPURL := URL;
      FRequest.Sign_Request(FHMACPost, FConsumer, nil);
      FHTTP.Request.Clear;
      FHTTP.HTTPOptions := [hoKeepOrigProtocol];
      try
        TCommunicator.SetProxy(FHTTP);
        Response := FHTTP.Post(URL, FRequest.Parameters);
      except
        on E: Exception do
        begin
          FExceptionMessage := E.Message;
          Exit;
        end;
      end;
      SetTokens(Response);
      Result := OAUTH_AUTHORIZE_URL + '?oauth_token=' + FOAuthToken;
    end
  finally
    // params.Free;
    // Request.Free;
  end;

end;

function TTwitterAuth.POSTCommand(URL: string; AParams: TStrings): TStrings;
var
  token: TOAuthToken;
  s: string;
begin
  FCriticalSection.Acquire;
  try
    Result := TStringList.Create;
    token := TOAuthToken.Create(FOAuthToken, FOAuthTokenSecret);
    try
      FRequest := FRequest.FromConsumerAndToken(FConsumer, token, URL);
      FRequest.HTTPURL := URL;
      if (AParams <> nil) then
      begin
        for s in AParams do
        begin
          FRequest.Parameters.Add(s);
        end;
      end;

      FRequest.Sign_Request(FHMACPost, Consumer, token);
      for s in FRequest.Parameters do
      begin
        // Result.Add(TOAuthUtil.urlEncodeRFC3986(s));
        Result.Add(s);
      end;
    finally
      token.Free;
    end;
  finally
    FCriticalSection.Leave;
  end;
end;

initialization

TwitterAuth := TTwitterAuth.Create;

finalization

TwitterAuth.Free;

end.
