unit tvURLShortener;

interface

uses
  Windows, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP, Forms, Controls, Dialogs, Graphics,
  ExtCtrls, Classes, SysUtils, Generics.Collections, SyncObjs, ActiveX, tvXMLBase, XMLIntf, XMLDoc,
  StrUtils, PerlRegEx;

type
  TURLShortener = class
  private
    FIdHttp: TIdHTTP;
    FXMLParser: TXMLParser;
  public
    ExceptionMessage: string;
    Status: string;

    function GetShortUrlLength: Integer;
    constructor Create;
    destructor Destroy; override;

    function GetShortUrlIsGd(ALongUrl: string): string;
    function GetShortUrlBitLy(ALongUrl: string): string;
    function GetShortUrlTinyURL(ALongUrl: string): string;
    function GetShortUrl(ALongUrl: string): string;
    function ShortenUrlInText(AText: string): string;
    function CanShortenUrlInText(AText: string): boolean;
  end;

var
  URLShortener: TURLShortener;

implementation

uses tvCommunicator, OAuth, tvUserConfig, tvFormLogs;
{ TURLShortener }

function TURLShortener.CanShortenUrlInText(AText: string): boolean;
var
  regex: TPerlRegEx;
  longUrl: string;
  match: boolean;
begin
  Result := false;
  if (UserConfig.URLShortenerConfig.Enabled) then
  begin
    regex := TPerlRegEx.Create(nil);
    regex.Options := [preSingleLine];
    try
      regex.regex := 'http:\/\/[-_.!~*\''()a-zA-Z0-9;\/?:\@&=+\$,%#]+';
      regex.Subject := AText;
      while (regex.MatchAgain) do
      begin
        if Length(regex.SubExpressions[0]) > GetShortUrlLength then
        begin
          Result := true;
          Exit;
        end;
      end;
    finally
      regex.Free;
    end;
  end;
end;

constructor TURLShortener.Create;
begin
  FIdHttp := TIdHTTP.Create(nil);
  FXMLParser := TXMLParser.Create;
end;

destructor TURLShortener.Destroy;
begin
  FXMLParser.Free;
  FIdHttp.Free;
  inherited;
end;

function TURLShortener.GetShortUrl(ALongUrl: string): string;
begin
  case UserConfig.URLShortenerConfig.URLShortenerSite of
    usBitLy:
      Result := GetShortUrlBitLy(ALongUrl);
    usTinyURL:
      Result := GetShortUrlTinyURL(ALongUrl);
    usIsGd:
      Result := GetShortUrlIsGd(ALongUrl);
  end;
end;

function TURLShortener.GetShortUrlBitLy(ALongUrl: string): string;
var
  url: string;
  Stream: TMemoryStream;
  XMLDoc: IXMLDocument;
  sLine: string;

begin
  ExceptionMessage := '';
  Status := '';

  Result := ALongUrl;
  TCommunicator.SetProxy(FIdHttp);
  Stream := TMemoryStream.Create;
  XMLDoc := TXMLDocument.Create(nil);
  try
    url := 'http://api.bit.ly/v3/shorten?' + 'login=' + OAuth.TOAuthUtil.urlEncodeRFC3986
      (UserConfig.URLShortenerConfig.UserName) + '&apiKey=' + UserConfig.URLShortenerConfig.APIKey + '&longUrl=' +
      OAuth.TOAuthUtil.urlEncodeRFC3986(ALongUrl) + '&format=xml';

    FIdHttp.Request.Clear;
    try
      FIdHttp.Get(url, Stream);
      // XMLǂݍ
      Stream.Seek(0, soBeginning);
      XMLDoc.LoadFromStream(Stream);
      FXMLParser.SetFromXML(XMLDoc);

      if (FXMLParser.Response.StatusCode = 200) then
      begin
        Result := FXMLParser.Response.ResponseData.url;
      end
      else
      begin
        Status := Format('StatusCode=%d StatusText=%s', [FXMLParser.Response.StatusCode,
          FXMLParser.Response.StatusText]);
        FormLogs.AddLogText(Status, Self.ClassName);
      end;
    except
      on E: Exception do
      begin
        ExceptionMessage := E.Message;
        FormLogs.AddLogText(E.Message, Self.ClassName);
      end;
    end;

  finally
    Stream.Free;
  end;
end;

function TURLShortener.GetShortUrlIsGd(ALongUrl: string): string;
var
  url: string;
  Stream: TMemoryStream;
  Reader: TStreamReader;
  sLine: string;
begin
  ExceptionMessage := '';
  Status := '';

  Result := ALongUrl;
  TCommunicator.SetProxy(FIdHttp);
  Stream := TMemoryStream.Create;
  Reader := TStreamReader.Create(Stream);
  try
    url := 'http://is.gd/api.php?longurl=' + OAuth.TOAuthUtil.urlEncodeRFC3986(ALongUrl);

    FIdHttp.Request.Clear;
    try
      FIdHttp.Get(url, Stream);
      Stream.Seek(0, soBeginning);

      sLine := Reader.ReadLine;
      if (StrUtils.StartsText('http:', sLine)) then
      begin
        Result := sLine;
      end;
    except
      on E: Exception do
      begin
        ExceptionMessage := E.Message;
        FormLogs.AddLogText(E.Message, Self.ClassName);
      end;
    end;

  finally
    Stream.Free;
    Reader.Free;
  end;
end;

function TURLShortener.GetShortUrlLength: Integer;
begin
  case UserConfig.URLShortenerConfig.URLShortenerSite of
    usBitLy:
      Result := 20;
    usTinyURL:
      Result := 25;
    usIsGd:
      Result := 18;
  end;
end;

function TURLShortener.GetShortUrlTinyURL(ALongUrl: string): string;
var
  url: string;
  Stream: TMemoryStream;
  Reader: TStreamReader;
  sLine: string;
begin
  ExceptionMessage := '';
  Status := '';

  Result := ALongUrl;
  TCommunicator.SetProxy(FIdHttp);
  Stream := TMemoryStream.Create;
  Reader := TStreamReader.Create(Stream);
  try
    url := 'http://tinyurl.com/api-create.php?url=' + OAuth.TOAuthUtil.urlEncodeRFC3986(ALongUrl);

    FIdHttp.Request.Clear;
    try
      FIdHttp.Get(url, Stream);
      Stream.Seek(0, soBeginning);

      sLine := Reader.ReadLine;
      if (StrUtils.StartsText('http:', sLine)) then
      begin
        Result := sLine;
      end;
    except
      on E: Exception do
      begin
        ExceptionMessage := E.Message;
        FormLogs.AddLogText(E.Message, Self.ClassName);
      end;
    end;

  finally
    Stream.Free;
    Reader.Free;
  end;

end;

function TURLShortener.ShortenUrlInText(AText: string): string;
var
  s: string;
  regex: TPerlRegEx;
  longUrl: string;
begin
  if (UserConfig.URLShortenerConfig.Enabled) then
  begin
    s := '';
    regex := TPerlRegEx.Create(nil);
    regex.Options := [preSingleLine];
    try
      regex.regex := '(.*?)(http:\/\/[-_.!~*\''()a-zA-Z0-9;\/?:\@&=+\$,%#]+)(.*)';
      while true do
      begin
        regex.Subject := AText;
        if (regex.match) then
        begin
          longUrl := regex.SubExpressions[2];
          if (Length(longUrl) > GetShortUrlLength) then
          begin
            longUrl := GetShortUrl(longUrl);
          end;
          s := s + regex.SubExpressions[1] + longUrl;
          AText := regex.SubExpressions[3];
        end
        else
        begin
          s := s + AText;
          break;
        end;
      end;
    finally
      regex.Free;
    end;

    Result := s;
  end
  else
  begin
    Result := AText;
  end;
end;

initialization

URLShortener := TURLShortener.Create;

finalization

URLShortener.Free;

end.
