unit tvFormCaption;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, tvXMLBase, tvRectView, ExtCtrls;

const
  HitBorderSize = 8; // ۂ̕\傫߂
  DispImageX = 4;
  DispImageY = 4;

  DispFollowIconX = 48 + 8;
  DispFollowIconY = 8;

  DispNameX = 48 + 8 + 20;
  DispNameY = 4;
  DispTextX = 4;
  DispTextY = 48 + 8;

  DispDateX = 4;
  DispDateY = 48 + 16;

type
  TFormCaption = class(TForm)
    Timer: TTimer;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseLeave(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
  private
    { Private 錾 }
    FStatus: TXMLStatus;
    FNextStatus: TXMLStatus;
    FRectViewName: TRectView;
    FRectViewText: TRectView;
    FRectViewDate: TRectView;
    FClosed: Boolean;
    FStatusChanged: Boolean;
    procedure SetStatus(const Value: TXMLStatus);
  public
    { Public 錾 }
    procedure UpdateUI;
    procedure UpdateCaption;
    procedure CreateParams(var Params: TCreateParams); override;

    property Status: TXMLStatus read FStatus write SetStatus;
  protected
    procedure WndProc(var Msg: TMessage); override;
  end;

var
  FormCaption: TFormCaption;

implementation

uses tvUserConfig, tvFormMain;
{$R *.dfm}
{ TFormCaption }

procedure TFormCaption.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.ExStyle := Params.ExStyle or WS_EX_TOOLWINDOW;
  Params.WndParent := 0;
end;

procedure TFormCaption.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  UserConfig.FormCaptionRect.Left := Left;
  UserConfig.FormCaptionRect.Top := Top;
  UserConfig.FormCaptionRect.Width := Width;
  UserConfig.FormCaptionRect.Height := Height;
  FClosed := true;
end;

procedure TFormCaption.FormCreate(Sender: TObject);
begin
  if (UserConfig.FormCaptionRect.Width <> 0) and (UserConfig.FormCaptionRect.Height <> 0) then
  begin
    Left := UserConfig.FormCaptionRect.Left;
    Top := UserConfig.FormCaptionRect.Top;
    Width := UserConfig.FormCaptionRect.Width;
    Height := UserConfig.FormCaptionRect.Height;
  end;
  // UpdateUI;
end;

procedure TFormCaption.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FRectViewName);
  FreeAndNil(FRectViewText);
  FreeAndNil(FRectViewDate);
end;

procedure TFormCaption.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  handled: Boolean;
begin
  handled := false;
  if Assigned(FRectViewText) then
  begin
    handled := FRectViewText.OnMouseDown(X - DispTextX, Y - DispTextY, Button, Shift);
  end;
  if (Not handled) and Assigned(FRectViewName) then
  begin
    handled := FRectViewName.OnMouseDown(X - DispNameX, Y - DispNameY, Button, Shift);
  end;
  if (Not handled) and Assigned(FRectViewDate) and Assigned(FRectViewText) then
  begin
    FRectViewDate.OnMouseDown(X - DispDateX, Y - DispDateY - FRectViewText.Height, Button, Shift);
  end;

end;

procedure TFormCaption.FormMouseLeave(Sender: TObject);
begin
  Screen.Cursor := crDefault;

end;

procedure TFormCaption.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  cs: TCursor;
  pt: TPoint;
begin
//  if (DispImageX + 48 > X) and (DispImageY + 48 > Y) then
//  begin
//    Screen.Cursor := crHandPoint;
//    Exit;
//  end;
  pt.X := X;
  pt.Y := Y;

  cs := crDefault;
  if Assigned(FRectViewText) then
  begin
    cs := FRectViewText.GetCursor(pt.X - DispTextX, pt.Y - DispTextY);
  end;
  if Assigned(FRectViewName) and (cs = crDefault) then
  begin
    cs := FRectViewName.GetCursor(pt.X - DispNameX, pt.Y - DispNameY);
  end;
  if Assigned(FRectViewDate) and Assigned(FRectViewText) and (cs = crDefault) then
  begin
    cs := FRectViewDate.GetCursor(pt.X - DispDateX, pt.Y - DispDateY - FRectViewText.Height);
  end;

  if (cs = crIBeam) then
  begin
    cs := Self.Cursor;
  end;
  Screen.Cursor := cs;
end;

procedure TFormCaption.FormPaint(Sender: TObject);
var
  rect: TRect;
  font: TFont;
  i: Integer;
  borderSize: Integer;
  Status: TXMLStatus;
  picture: TPicture;
  procedure DrawText(ofsx, ofsy: Integer; ABorder: Boolean);
  begin
    if Assigned(FRectViewText) then
    begin
      rect.Left := DispTextX + ofsx;
      rect.Top := DispTextY + ofsy;
      rect.Right := Width;
      rect.Bottom := Height;

      if (ABorder) then
      begin
        Canvas.font.Color := UserConfig.CaptionFont.BorderColor;
      end
      else
      begin
        Canvas.font.Color := UserConfig.CaptionFont.Color;
      end;
      FRectViewText.Canvas := Canvas;
      FRectViewText.Draw(rect);
    end;
  end;
  procedure DrawUserName(ofsx, ofsy: Integer; ABorder: Boolean);
  begin
    if Assigned(FRectViewName) then
    begin
      rect.Left := DispNameX + ofsx;
      rect.Top := DispNameY + ofsy;
      rect.Right := Width;
      rect.Bottom := Height;

      if (ABorder) then
      begin
        Canvas.font.Color := UserConfig.CaptionFontUserName.BorderColor;
      end
      else
      begin
        Canvas.font.Color := UserConfig.CaptionFontUserName.Color;
      end;

      FRectViewName.Canvas := Canvas;
      FRectViewName.Draw(rect);
    end;
  end;
  procedure DrawUserDate(ofsx, ofsy: Integer; ABorder: Boolean);
  begin
    if Assigned(FRectViewDate) and Assigned(FRectViewText) then
    begin
      rect.Left := DispDateX + ofsx;
      rect.Top := DispDateY + ofsy + FRectViewText.Height;
      rect.Right := Width;
      rect.Bottom := Height;

      if (ABorder) then
      begin
        Canvas.font.Color := UserConfig.CaptionFontDateTime.BorderColor;
      end
      else
      begin
        Canvas.font.Color := UserConfig.CaptionFontDateTime.Color;
      end;

      FRectViewDate.Canvas := Canvas;
      FRectViewDate.Draw(rect);
    end;
  end;

begin
  if (UserConfig.CaptionType <> ctCaptionNormal) and (UserConfig.CaptionType <> ctCaptionWindow) then
  begin
    Exit;
  end;

  if (UserConfig.CaptionType = ctCaptionWindow) then
  begin
    rect.Top := 0;
    rect.Left := 0;
    rect.Right := Width;
    rect.Bottom := Height;
    Canvas.Pen.Width := 2;
    Canvas.Pen.Color := clBlack;
    Canvas.Brush.Color := clLtGray;
    Canvas.FrameRect(rect);
  end;
  if Assigned(FStatus) then
  begin
    Status := FStatus;
    if Assigned(Status.RetweetedStatus) then
    begin
      Status := Status.RetweetedStatus;
    end;

    if Assigned(Status.User.ProfileImage) then
    begin
      Canvas.Draw(DispImageX, DispImageY, Status.User.ProfileImage);
    end;

    Canvas.Brush.Style := bsClear;
    borderSize := UserConfig.CaptionFont.borderSize;

    font := UserConfig.CaptionFont.CreateFont;
    Canvas.font.Assign(font);

    for i := 0 to borderSize - 1 do
    begin
      DrawText(i, i - borderSize, true);
      DrawText(borderSize - i, i, true);
      DrawText(-i, borderSize - i, true);
      DrawText(-borderSize + i, -i, true);
    end;

    DrawText(0, 0, false);
    font.Free;

    borderSize := UserConfig.CaptionFontUserName.borderSize;
    font := UserConfig.CaptionFontUserName.CreateFont;
    Canvas.font.Assign(font);

    picture := nil;
    case Status.User.FollowingIndex of
      1:
        picture := FormMainTweetVoice.ImageFollowedByMe.picture;
      2:
        picture := FormMainTweetVoice.ImageFollowingMe.picture;
      3:
        picture := FormMainTweetVoice.ImageFollowingEachOther.picture;
    else
      picture := FormMainTweetVoice.ImageNoFollow.picture;
    end;
    if Assigned(picture) then
    begin
      Canvas.Draw(DispFollowIconX, DispFollowIconY + (Canvas.TextHeight(' ') div 2) - 10, picture.Graphic);
    end;

    for i := 0 to borderSize - 1 do
    begin
      DrawUserName(i, i - borderSize, true);
      DrawUserName(borderSize - i, i, true);
      DrawUserName(-i, borderSize - i, true);
      DrawUserName(-borderSize + i, -i, true);
    end;

    DrawUserName(0, 0, false);
    font.Free;

    borderSize := UserConfig.CaptionFontDateTime.borderSize;
    font := UserConfig.CaptionFontDateTime.CreateFont;
    Canvas.font.Assign(font);
    for i := 0 to borderSize - 1 do
    begin
      DrawUserDate(i, i - borderSize, true);
      DrawUserDate(borderSize - i, i, true);
      DrawUserDate(-i, borderSize - i, true);
      DrawUserDate(-borderSize + i, -i, true);
    end;

    DrawUserDate(0, 0, false);
    font.Free;
  end;
end;

procedure TFormCaption.FormResize(Sender: TObject);
begin
  Invalidate;
  UpdateCaption;
end;

procedure TFormCaption.SetStatus(const Value: TXMLStatus);
begin
  FNextStatus := Value;
  FStatusChanged := true;
end;

procedure TFormCaption.TimerTimer(Sender: TObject);
var
  alpha: Integer;
begin
  if (FStatusChanged) then
  begin
    AlphaBlend := true;
    if (FNextStatus <> FStatus) then
    begin
      alpha := AlphaBlendValue - 80;
      if (alpha < 0) then
      begin
        AlphaBlendValue := 0;
        FStatus := FNextStatus;
        UpdateCaption;
        Repaint;
      end
      else
      begin
        AlphaBlendValue := alpha;
      end;
    end
    else
    begin
      alpha := AlphaBlendValue + 80;
      if (alpha > 255) then
      begin
        AlphaBlendValue := 255;
        FStatusChanged := false;
      end
      else
      begin
        AlphaBlendValue := alpha;
      end;
    end;
  end
  else
  begin
    AlphaBlend := false;
    // Repaint;
  end;
end;

procedure TFormCaption.UpdateCaption;
var
  font: TFont;
  Status: TXMLStatus;
  RetweetStatus: TXMLStatus;
  urlTextItem: TUrlTextItem;
  plainTextItem: TPlainTextItem;

begin
  FreeAndNil(FRectViewName);
  FreeAndNil(FRectViewText);
  FreeAndNil(FRectViewDate);

  if Assigned(FStatus) then
  begin
    Status := FStatus;
    RetweetStatus := nil;
    if Assigned(Status.RetweetedStatus) then
    begin
      RetweetStatus := Status;
      Status := Status.RetweetedStatus;
    end;

    font := UserConfig.CaptionFontUserName.CreateFont;
    FRectViewName := Status.User.CreateUserRectView(false, RetweetStatus);
    FRectViewName.Clip := false;
    FRectViewName.Canvas := Canvas;
    FRectViewName.Canvas.font := font;
    FRectViewName.SetFontStyleToAllItems(font.Style);
    FRectViewName.ArrangeLayout(Width - DispNameX - UserConfig.CaptionFontUserName.borderSize * 2);

    font.Free;

    font := UserConfig.CaptionFont.CreateFont;
    FRectViewText := TRectView.Create;
    FRectViewText.Clip := false;
    FRectViewText.Text := Status.Text;
    FRectViewText.CanSelect := false;
    FRectViewText.Canvas := Canvas;
    FRectViewText.Canvas.font := font;
    FRectViewText.SetFontStyleToAllItems(font.Style);
    // FRectViewText.Items.Add(TBreakItem.Create(FRectViewText));
    FRectViewText.ArrangeLayout(Width - DispTextX - UserConfig.CaptionFont.borderSize * 2);
    font.Free;

    font := UserConfig.CaptionFontDateTime.CreateFont;

    FRectViewDate := TRectView.Create;
    FRectViewDate.Clip := false;
    FRectViewDate.Canvas := Canvas;
    FRectViewDate.Canvas.font := font;
    urlTextItem := TUrlTextItem.Create(FRectViewDate);
    urlTextItem.Text := GetDateString(Status.LocalTimeCreatedAt, 'ja');
    urlTextItem.Url := Status.GetTweetUrl;
    FRectViewDate.Items.Add(urlTextItem);

    plainTextItem := TPlainTextItem.Create(FRectViewDate);
    plainTextItem.Text := ' ';
    FRectViewDate.Items.Add(plainTextItem);

    Status.AddSourceToRectView(FRectViewDate);

    plainTextItem := TPlainTextItem.Create(FRectViewDate);
    plainTextItem.Text := '';
    FRectViewDate.Items.Add(plainTextItem);

    FRectViewDate.SetFontStyleToAllItems(font.Style);
    FRectViewDate.ArrangeLayout(Width - DispDateX - UserConfig.CaptionFont.borderSize * 2);

    font.Free;

  end;
  Invalidate;
end;

procedure TFormCaption.UpdateUI;
begin
  if (UserConfig.CaptionType <> ctCaptionNormal) and (UserConfig.CaptionType <> ctCaptionWindow) then
  begin
    Visible := false;
    Exit;
  end;
  SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE + SWP_NOSIZE);
  Visible := true;
  if (UserConfig.CaptionType = ctCaptionNormal) then
  begin
    Self.Color := RGB(0, 0, 0);
    Self.TransparentColor := true;
    Self.TransparentColorValue := RGB(0, 0, 0);
  end
  else
  begin
    Self.Color := clBtnFace;
    Self.TransparentColor := false;
  end;
end;

procedure TFormCaption.WndProc(var Msg: TMessage);
var
  pt: TPoint;
  X, Y: SmallInt;
  hit, hitY: Integer;

  function IsTopMost(AHandle: HWND): Boolean;
  begin
    Result := ((GetWindowLong(AHandle, GWL_EXSTYLE) and WS_EX_TOPMOST) <> 0);
  end;

begin
  case Msg.Msg of
    WM_NCHITTEST:
      begin
        X := SmallInt(Cardinal(Msg.LParam) and $FFFF);
        Y := SmallInt(Cardinal(Msg.LParam) shr 16);
        pt.X := X;
        pt.Y := Y;
        pt := Self.ScreenToClient(pt);

        if (pt.X < 48 + DispImageX) and (pt.Y < 48 + DispImageY) then
        begin
          hitY := HTCAPTION;
        end
        else if Assigned(FStatus) then
        begin
          hitY := HTCLIENT;
        end
        else
        begin
          hitY := HTCAPTION;
        end;

        if (pt.Y <= HitBorderSize) then
        begin
          hitY := HTTOP;
        end
        else if (pt.Y >= Height - HitBorderSize) then
        begin
          hitY := HTBOTTOM;
        end;

        if (pt.X <= HitBorderSize) then
        begin
          if (hitY = HTBOTTOM) then
          begin
            hit := HTBOTTOMLEFT;
          end
          else if (hitY = HTTOP) then
          begin
            hit := HTTOPLEFT;
          end
          else
          begin
            hit := HTLEFT;
          end;
        end
        else if (pt.X >= Width - HitBorderSize) then
        begin
          if (hitY = HTBOTTOM) then
          begin
            hit := HTBOTTOMRIGHT;
          end
          else if (hitY = HTTOP) then
          begin
            hit := HTTOPRIGHT;
          end
          else
          begin
            hit := HTRIGHT;
          end;
        end
        else
        begin
          hit := hitY;
        end;
        Msg.Result := hit;
      end;
    WM_WINDOWPOSCHANGING:
      begin
        inherited WndProc(Msg);
        if (Not FClosed) and (Not IsTopMost(Handle)) then
        begin
          SetWindowPos(Handle, HWND_TOPMOST, 0, 0, 0, 0,
            SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE or SWP_NOOWNERZORDER);
        end;

      end;
  else
    inherited WndProc(Msg);

  end;
end;

end.
