unit tvFormCaptionFontOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, tvUserConfig, ExtCtrls, ComCtrls;

type
  TFormCaptionFontOptions = class(TForm)
    ButtonCancel: TButton;
    ButtonOK: TButton;
    PaintBoxPreview: TPaintBox;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxFontNames: TComboBox;
    EditBorderSize: TEdit;
    Label3: TLabel;
    ComboBoxFontSize: TComboBox;
    UpDownBorderSize: TUpDown;
    CheckBoxBold: TCheckBox;
    CheckBoxItalic: TCheckBox;
    Label4: TLabel;
    Label5: TLabel;
    PanelFontColor: TPanel;
    PanelBorderColor: TPanel;
    ColorDialog: TColorDialog;
    procedure FormCreate(Sender: TObject);
    procedure PanelFontColorClick(Sender: TObject);
    procedure PanelBorderColorClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
    procedure PaintBoxPreviewPaint(Sender: TObject);
    procedure ComboBoxFontNamesChange(Sender: TObject);
    procedure UpDownBorderSizeChanging(Sender: TObject; var AllowChange: Boolean);
  private
    { Private 錾 }
  public
    { Public 錾 }
    procedure GetFromUserConfig(ACaptionFont: TCaptionFont);
    procedure SetToUserConfig(ACaptionFont: TCaptionFont);
  end;

var
  FormCaptionFontOptions: TFormCaptionFontOptions;

implementation

{$R *.dfm}
{ TFormCaptionFontOptions }

procedure TFormCaptionFontOptions.ButtonOKClick(Sender: TObject);
begin
  Close;
  ModalResult := mrOK;
end;

procedure TFormCaptionFontOptions.ComboBoxFontNamesChange(Sender: TObject);
begin
  PaintBoxPreview.Invalidate;
end;

procedure TFormCaptionFontOptions.FormCreate(Sender: TObject);
begin
  ComboBoxFontNames.Items.Assign(Screen.Fonts);
end;

procedure TFormCaptionFontOptions.GetFromUserConfig(ACaptionFont: TCaptionFont);
begin
  ComboBoxFontNames.Text := ACaptionFont.Name;
  ComboBoxFontSize.Text := IntToStr(ACaptionFont.Size);
  PanelFontColor.Color := ACaptionFont.Color;
  PanelBorderColor.Color := ACaptionFont.BorderColor;
  CheckBoxBold.Checked := ACaptionFont.IsBold;
  CheckBoxItalic.Checked := ACaptionFont.IsItalic;
  UpDownBorderSize.Position := ACaptionFont.BorderSize;
end;

procedure TFormCaptionFontOptions.PaintBoxPreviewPaint(Sender: TObject);
var
  i: Integer;
  BorderSize: Integer;
  rect: TRect;
  tf: TTextFormat;
  s: string;
  procedure DrawText(ofsx, ofsy: Integer; ABorder: Boolean);
  begin
    rect.Left := ofsx + BorderSize;
    rect.Top := ofsy + BorderSize;
    rect.Right := PaintBoxPreview.Width + ofsx - 10;
    rect.Bottom := PaintBoxPreview.Height;

    if (ABorder) then
    begin
      PaintBoxPreview.Canvas.Font.Color := PanelBorderColor.Color;
    end
    else
    begin
      PaintBoxPreview.Canvas.Font.Color := PanelFontColor.Color;
    end;
    // PaintBoxPreview.Canvas.TextRect(rect, ofsx, ofsy, 'vr[ 123 ABC');
    // PaintBoxPreview.Canvas.TextRect(rect, 'vr[ 123 ABC', [tfNoPrefix]);
    tf := [tfWordBreak , tfNoClip];
    s := 'vr[ 123 ABC';
    PaintBoxPreview.Canvas.TextRect(rect, s, tf);
  end;

begin
  BorderSize := UpDownBorderSize.Position;
  PaintBoxPreview.Canvas.Font.Name := ComboBoxFontNames.Text;
  PaintBoxPreview.Canvas.Font.Size := StrToIntDef(ComboBoxFontSize.Text, 14);
  PaintBoxPreview.Canvas.Font.Style := [];
  PaintBoxPreview.Canvas.Brush.Style := bsClear;
  if (CheckBoxBold.Checked) then
  begin
    PaintBoxPreview.Canvas.Font.Style := [fsBold];
  end;
  if (CheckBoxItalic.Checked) then
  begin
    PaintBoxPreview.Canvas.Font.Style := PaintBoxPreview.Canvas.Font.Style + [fsItalic];
  end;

  for i := 0 to BorderSize - 1 do
  begin
    DrawText(i, i - BorderSize, true);
    DrawText(BorderSize - i, i, true);
    DrawText(-i, BorderSize - i, true);
    DrawText(-BorderSize + i, -i, true);
  end;

  //PaintBoxPreview.Canvas.Brush.Color := PanelBorderColor.Color;
  //PaintBoxPreview.Canvas.Brush.Style := bsSolid;
  DrawText(0, 0, false);

end;

procedure TFormCaptionFontOptions.PanelBorderColorClick(Sender: TObject);
begin
  ColorDialog.Color := PanelBorderColor.Color;
  if (ColorDialog.Execute) then
  begin
    PanelBorderColor.Color := ColorDialog.Color;
    PaintBoxPreview.Invalidate;
  end;
end;

procedure TFormCaptionFontOptions.PanelFontColorClick(Sender: TObject);
begin
  ColorDialog.Color := PanelFontColor.Color;
  if (ColorDialog.Execute) then
  begin
    PanelFontColor.Color := ColorDialog.Color;
    PaintBoxPreview.Invalidate;
  end;
end;

procedure TFormCaptionFontOptions.SetToUserConfig(ACaptionFont: TCaptionFont);
begin
  ACaptionFont.Name := ComboBoxFontNames.Text;
  ACaptionFont.Size := StrToIntDef(ComboBoxFontSize.Text, 18);
  ACaptionFont.Color := PanelFontColor.Color;
  ACaptionFont.BorderColor := PanelBorderColor.Color;
  ACaptionFont.IsBold := CheckBoxBold.Checked;
  ACaptionFont.IsItalic := CheckBoxItalic.Checked;
  ACaptionFont.BorderSize := UpDownBorderSize.Position;
end;

procedure TFormCaptionFontOptions.UpDownBorderSizeChanging(Sender: TObject; var AllowChange: Boolean);
begin
  PaintBoxPreview.Invalidate;
end;

end.
