unit tvFormLogin;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, tvUserConfig;

type
  TFormLogin = class(TForm)
    ComboBoxScreenName: TComboBox;
    Label1: TLabel;
    ButtonOK: TButton;
    ButtonCancel: TButton;
    ButtonRemoveUser: TButton;
    ButtonOptions: TButton;
    CheckBoxHideFormLoginStartup: TCheckBox;
    ButtonLogin: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ComboBoxScreenNameChange(Sender: TObject);
    procedure ButtonRemoveUserClick(Sender: TObject);
    procedure ButtonOptionsClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
    procedure ButtonLoginClick(Sender: TObject);
  private
    { Private 錾 }
    function GetCurrentUserAuth: TUserAuth;
    procedure UpdateUI;
    procedure UpdateComboBox;
    procedure SetHideFormLoginStartup(const Value: Boolean);
    function GetFormLoginStartup: Boolean;
  public
    { Public 錾 }
    property HideFormLoginStartup: Boolean read GetFormLoginStartup write SetHideFormLoginStartup;
  end;

var
  FormLogin: TFormLogin;

implementation

uses tvOAuth, tvFormOptions, tvFormPinCode;
{$R *.dfm}

procedure TFormLogin.ButtonCancelClick(Sender: TObject);
begin
  UserConfig.ShowFormLoginStartUp := Not FormLogin.HideFormLoginStartup;
  UserConfig.Save;
  ModalResult := mrCancel;
end;

procedure TFormLogin.ButtonLoginClick(Sender: TObject);
var
  url: string;
  ua: TUserAuth;
  userAuth: TUserAuth;
  i: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    url := TwitterAuth.GetTwitterAppAuthURL;
    if (url = '') then
    begin
      MessageDlg('FURL擾Ɏs܂B'#$d#$a + TwitterAuth.ExceptionMessage, mtError, [mbOK], 0);
      Exit;
    end;
    FormPinCode.Initialize(url);
    if (FormPinCode.ShowModal = mrOK) then
    begin
      if (TwitterAuth.Authorize(FormPinCode.PinCode)) then
      begin
        userAuth := nil;
        for i := 0 to UserConfig.UserOAuths.Count - 1 do
        begin
          ua := UserConfig.UserOAuths.Items[i] as TUserAuth;
          if (string(ua.UserName) = TwitterAuth.ScreenName) then
          begin
            userAuth := ua;
            break;
          end;
        end;

        if Not Assigned(userAuth) then
        begin
          userAuth := UserConfig.UserOAuths.Add as TUserAuth;
        end;
        userAuth.UserName := AnsiString(TwitterAuth.ScreenName);
        userAuth.DisplayName := string(userAuth.UserName);

        userAuth.OAuthToken := AnsiString(TwitterAuth.OAuthToken);
        userAuth.OAuthTokenSecret := AnsiString(TwitterAuth.OAuthTokenSecret);
        UserConfig.DefaultUserName := string(userAuth.UserName);
        UserConfig.ShowFormLoginStartUp := Not FormLogin.HideFormLoginStartup;
        UserConfig.Save;

        UpdateComboBox;
        Close;
        ModalResult := mrOK;
      end
      else
      begin
        MessageDlg('OCɎs܂B'#$d#$a + TwitterAuth.ExceptionMessage, mtError, [mbOK], 0);
      end;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormLogin.ButtonOKClick(Sender: TObject);
var
  url: string;
  ua: TUserAuth;
  userAuth: TUserAuth;
  i: Integer;
begin
  userAuth := nil;
  for i := 0 to UserConfig.UserOAuths.Count - 1 do
  begin
    ua := UserConfig.UserOAuths.Items[i] as TUserAuth;
    if (string(ua.UserName) = ComboBoxScreenName.Text) then
    begin
      userAuth := ua;
      break;
    end;
  end;
  Screen.Cursor := crHourGlass;
  try
    if Assigned(userAuth) then
    begin

      TwitterAuth.OAuthToken := string(userAuth.OAuthToken);
      TwitterAuth.OAuthTokenSecret := string(userAuth.OAuthTokenSecret);
      if (TwitterAuth.CheckAuthToken) then
      begin
        UserConfig.DefaultUserName := string(userAuth.UserName);
        UserConfig.ShowFormLoginStartUp := Not FormLogin.HideFormLoginStartup;
        UserConfig.Save;
        Close;
        ModalResult := mrOK;
      end
      else
      begin
        MessageDlg('OCɎs܂B', mtWarning, [mbOK], 0);
        UserConfig.UserOAuths.Delete(userAuth.Index);
        UpdateUI;
      end;
    end;
  finally
    Screen.Cursor := crDefault;
  end;

end;

procedure TFormLogin.ButtonOptionsClick(Sender: TObject);
begin
  FormOptions.GetFromUserConfig;
  FormOptions.PageControl.TabIndex := 1;
  if (FormOptions.ShowModal = mrOK) then
  begin
    FormOptions.SetToUserConfig;
  end;
  FormOptions.PageControl.TabIndex := 0;
end;

procedure TFormLogin.ButtonRemoveUserClick(Sender: TObject);
var
  ua: TUserAuth;
  i: Integer;
begin
  ua := GetCurrentUserAuth;
  if (Assigned(ua)) then
  begin
    UserConfig.UserOAuths.Delete(ua.Index);
    // ua.Free;
  end;
  ComboBoxScreenName.Items.Clear;
  for i := 0 to UserConfig.UserOAuths.Count - 1 do
  begin
    ua := UserConfig.UserOAuths.Items[i] as TUserAuth;
    ComboBoxScreenName.Items.Add(string(ua.UserName));
  end;
  ComboBoxScreenName.ItemIndex := -1;
  UpdateUI;
end;

procedure TFormLogin.ComboBoxScreenNameChange(Sender: TObject);
begin
  UpdateUI;
end;

procedure TFormLogin.FormCreate(Sender: TObject);
begin
  UpdateComboBox;
end;

function TFormLogin.GetCurrentUserAuth: TUserAuth;
var
  s: string;
  ua: TUserAuth;
  i: Integer;
begin
  Result := nil;
  s := ComboBoxScreenName.Text;
  for i := 0 to UserConfig.UserOAuths.Count - 1 do
  begin
    ua := UserConfig.UserOAuths.Items[i] as TUserAuth;
    if (s = string(ua.UserName)) then
    begin
      Result := ua;
      break;
    end;
  end;
end;

function TFormLogin.GetFormLoginStartup: Boolean;
begin
  Result := CheckBoxHideFormLoginStartup.Checked;
end;

procedure TFormLogin.SetHideFormLoginStartup(const Value: Boolean);
begin
  CheckBoxHideFormLoginStartup.Checked := Value;
end;

procedure TFormLogin.UpdateComboBox;
var
  ua: TUserAuth;
  i: Integer;
  idx: Integer;
begin
  idx := -1;
  for i := 0 to UserConfig.UserOAuths.Count - 1 do
  begin
    ua := UserConfig.UserOAuths.Items[i] as TUserAuth;
    ComboBoxScreenName.Items.Add(string(ua.UserName));
    if (ua.UserName = UserConfig.DefaultUserName) then
    begin
      idx := i;
    end;
  end;
  ComboBoxScreenName.ItemIndex := idx;
  UpdateUI;
end;

procedure TFormLogin.UpdateUI;
begin
  if (ComboBoxScreenName.ItemIndex >= 0) then
  begin
    ButtonRemoveUser.Enabled := true;
    ButtonOK.Enabled := true;
  end
  else
  begin
    ButtonRemoveUser.Enabled := false;
    ButtonOK.Enabled := false;
  end;
end;

end.
