unit tvFormLogs;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, VirtualTrees, Menus, Clipbrd;

const
  MAX_LOG_LINE_COUNT = 5000;

type
  TFormLogs = class(TForm)
    VirtualTreeView: TVirtualStringTree;
    PopupMenu: TPopupMenu;
    popMenuCopySelectedItems: TMenuItem;
    menuItemSelectAll: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure VirtualTreeViewGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
      TextType: TVSTTextType; var CellText: string);
    procedure FormDestroy(Sender: TObject);
    procedure menuItemSelectAllClick(Sender: TObject);
    procedure popMenuCopySelectedItemsClick(Sender: TObject);
  private
    { Private 錾 }
    FFormClosing: boolean;
  public
    { Public 錾 }
    procedure AddLogText(ALogText: string; AClassName: string = '');
  end;

  TTreeData = record
    DateTimeStr: ShortString;
    LogText: ShortString;
    ClassName: ShortString;
    pNodeData: Pointer;
  end;

  PTreeData = ^TTreeData;

var
  FormLogs: TFormLogs;

implementation

{$R *.dfm}
{ TFormLogs }

procedure TFormLogs.AddLogText(ALogText: string; AClassName: string);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
  rect: TRect;
  INode: PVirtualNode;
  i, Count: Integer;
begin
  if FFormClosing then
    Exit;
  Node := VirtualTreeView.AddChild(nil);

  if (VirtualTreeView.RootNode.ChildCount > MAX_LOG_LINE_COUNT) then
  begin
    VirtualTreeView.DeleteNode(VirtualTreeView.RootNode.FirstChild);
  end;

  dataPtr := VirtualTreeView.GetNodeData(Node);
  dataPtr.DateTimeStr := ShortString(FormatDateTime('mm/dd hh:nn:ss', Now));
  dataPtr.LogText := ShortString(ALogText);
  dataPtr.ClassName := ShortString(AClassName);
  rect := VirtualTreeView.GetDisplayRect(Node, 0, true);
  if (rect.Top + Integer(VirtualTreeView.DefaultNodeHeight) * 3 > VirtualTreeView.Height) then
  begin
    Count := VirtualTreeView.Height div Integer(VirtualTreeView.DefaultNodeHeight);
    INode := Node;
    for i := 0 to Count - 4 do
    begin
      if Not Assigned(INode.PrevSibling) then
      begin
        break;
      end;
      INode := INode.PrevSibling;
    end;
    VirtualTreeView.TopNode := INode;
  end;
end;

procedure TFormLogs.FormCreate(Sender: TObject);
begin
  VirtualTreeView.NodeDataSize := Sizeof(TTreeData);
end;

procedure TFormLogs.FormDestroy(Sender: TObject);
begin
  FFormClosing := true;
  VirtualTreeView.Clear;
end;

procedure TFormLogs.menuItemSelectAllClick(Sender: TObject);
begin
  VirtualTreeView.SelectAll(false);
end;

procedure TFormLogs.popMenuCopySelectedItemsClick(Sender: TObject);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
  sb: TStringBuilder;
begin
  sb := TStringBuilder.Create;
  try
    Node := VirtualTreeView.RootNode.FirstChild;
    while (Node <> nil) do
    begin
      dataPtr := VirtualTreeView.GetNodeData(Node);
      if (VirtualTreeView.Selected[Node]) then
      begin
        sb.AppendLine(Format('%s %s %s', [dataPtr.DateTimeStr, dataPtr.ClassName, dataPtr.LogText]));
      end;
      Node := Node.NextSibling;
    end;
    Clipboard.AsText := sb.ToString;
  finally
    sb.Free;
  end;
end;

procedure TFormLogs.VirtualTreeViewGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var CellText: string);
var
  dataPtr: PTreeData;
begin
  dataPtr := VirtualTreeView.GetNodeData(Node);
  case Column of
    0:
      CellText := string(dataPtr.DateTimeStr);
    1:
      CellText := string(dataPtr.ClassName);
    2:
      CellText := string(dataPtr.LogText);
  end;
end;

end.
