unit tvFormOptions;

interface

uses
  Windows, MMSYSTEM, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, StdCtrls, utFunctions, tvUserConfig;

type
  TFormOptions = class(TForm)
    ButtonOK: TButton;
    ButtonCancel: TButton;
    PageControl: TPageControl;
    TabSheetView: TTabSheet;
    TabSheetNet: TTabSheet;
    Label1: TLabel;
    EditAutoUpdateSec: TEdit;
    UpDownAutoUpdateSecHome: TUpDown;
    CheckBoxUseProxy: TCheckBox;
    PanelProxy: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    EditProxyHost: TEdit;
    EditProxyPort: TEdit;
    UpDownProxyPort: TUpDown;
    TabSheetSpeech: TTabSheet;
    ComboBoxListViewFontName: TComboBox;
    Label4: TLabel;
    EditListViewFontSize: TEdit;
    UpDownListViewFontSize: TUpDown;
    Label5: TLabel;
    Edit1: TEdit;
    UpDownAutoUpdateSecUser: TUpDown;
    Label6: TLabel;
    Edit2: TEdit;
    UpDownAutoUpdateSecReplies: TUpDown;
    Label7: TLabel;
    Edit3: TEdit;
    UpDownAutoUpdateSecDirectMessages: TUpDown;
    Label8: TLabel;
    Edit4: TEdit;
    UpDownAutoUpdateSecFavorites: TUpDown;
    Label9: TLabel;
    Edit5: TEdit;
    UpDownAutoUpdateSecPublicList: TUpDown;
    Label10: TLabel;
    Edit6: TEdit;
    UpDownAutoUpdateSecPrivateList: TUpDown;
    Edit7: TEdit;
    UpDownAutoUpdateSecSearch: TUpDown;
    Label11: TLabel;
    Label12: TLabel;
    Edit8: TEdit;
    UpDownRequestStatusesCount: TUpDown;
    ListViewSpeeches: TListView;
    ButtonEditSpeechConfig: TButton;
    Label13: TLabel;
    ButtonCaptionFontText: TButton;
    ButtonCaptionFontUserName: TButton;
    TabSheetPost: TTabSheet;
    GroupBoxURLShortener: TGroupBox;
    ComboBoxURLShortenerSite: TComboBox;
    CheckBoxComplementScreenName: TCheckBox;
    TabSheetSound: TTabSheet;
    CheckBoxPlaySoundBeforeSpeech: TCheckBox;
    PanelPlaySound: TPanel;
    ButtonPlaySoundFile: TButton;
    ButtonOpenSoundFile: TButton;
    EditSoundFileName: TEdit;
    Label16: TLabel;
    CheckBoxSynchPlaySound: TCheckBox;
    OpenDialogWavFile: TOpenDialog;
    PanelURLShorten: TPanel;
    Label14: TLabel;
    EditURLShortenerUserName: TEdit;
    Label15: TLabel;
    EditURLShortenerPassword: TEdit;
    LinkLabelAPIKey: TLinkLabel;
    EditURLShortenerAPIKey: TEdit;
    ButtonCaptionFontDateTime: TButton;
    procedure ButtonOKClick(Sender: TObject);
    procedure ButtonCancelClick(Sender: TObject);
    procedure CheckBoxUseProxyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonEditSpeechConfigClick(Sender: TObject);
    procedure ListViewSpeechesSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure ButtonCaptionFontTextClick(Sender: TObject);
    procedure ButtonCaptionFontUserNameClick(Sender: TObject);
    procedure LinkLabelAPIKeyClick(Sender: TObject);
    procedure CheckBoxPlaySoundBeforeSpeechClick(Sender: TObject);
    procedure ButtonOpenSoundFileClick(Sender: TObject);
    procedure ButtonPlaySoundFileClick(Sender: TObject);
    procedure ComboBoxURLShortenerSiteChange(Sender: TObject);
    procedure ButtonCaptionFontDateTimeClick(Sender: TObject);
  private
    { Private 錾 }

    FSpeechConfigs: TCollection;
    FCaptionFontText: TCaptionFont;
    FCaptionFontUserName: TCaptionFont;
    FCaptionFontDateTime: TCaptionFont;
    procedure UpdateUI;
    procedure UpdateListViewSpeeches;
  public
    { Public 錾 }
    procedure GetFromUserConfig;
    procedure SetToUserConfig;
  end;

var
  FormOptions: TFormOptions;

implementation

uses tvFormSpeechConfig, tvFormCaption, tvFormCaptionFontOptions;
{$R *.dfm}

procedure TFormOptions.ButtonCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TFormOptions.ButtonCaptionFontDateTimeClick(Sender: TObject);
begin
  FormCaptionFontOptions.GetFromUserConfig(FCaptionFontDateTime);
  if (FormCaptionFontOptions.ShowModal = mrOK) then
  begin
    FormCaptionFontOptions.SetToUserConfig(FCaptionFontDateTime);
  end;
end;

procedure TFormOptions.ButtonCaptionFontTextClick(Sender: TObject);
begin
  FormCaptionFontOptions.GetFromUserConfig(FCaptionFontText);
  if (FormCaptionFontOptions.ShowModal = mrOK) then
  begin
    FormCaptionFontOptions.SetToUserConfig(FCaptionFontText);
  end;
end;

procedure TFormOptions.ButtonCaptionFontUserNameClick(Sender: TObject);
begin
  FormCaptionFontOptions.GetFromUserConfig(FCaptionFontUserName);
  if (FormCaptionFontOptions.ShowModal = mrOK) then
  begin
    FormCaptionFontOptions.SetToUserConfig(FCaptionFontUserName);
  end;
end;

procedure TFormOptions.ButtonEditSpeechConfigClick(Sender: TObject);
var
  Item: TListItem;
  speechConfig: TSpeechConfig;
begin
  Item := ListViewSpeeches.Selected;
  if Assigned(Item) then
  begin
    speechConfig := TSpeechConfig(Item.Data);
    FormSpeechConfig.GetFromUserConfig(speechConfig);
    if (FormSpeechConfig.ShowModal = mrOK) then
    begin
      FormSpeechConfig.SetToUserConfig(speechConfig);
      UpdateListViewSpeeches;
    end;
  end;
end;

procedure TFormOptions.ButtonOKClick(Sender: TObject);
begin
  Close;
  ModalResult := mrOK;
end;

procedure TFormOptions.ButtonOpenSoundFileClick(Sender: TObject);
begin
  OpenDialogWavFile.FileName := EditSoundFileName.Text;
  if (OpenDialogWavFile.Execute) then
  begin
    EditSoundFileName.Text := OpenDialogWavFile.FileName;
  end;
end;

procedure TFormOptions.ButtonPlaySoundFileClick(Sender: TObject);
var
  soundFileName: WideString;
begin
  if (FileExists(EditSoundFileName.Text)) then
  begin
    soundFileName := EditSoundFileName.Text;
    PlaySound(PWideChar(soundFileName), 0, SND_FILENAME or SND_ASYNC);
  end;
end;

procedure TFormOptions.CheckBoxPlaySoundBeforeSpeechClick(Sender: TObject);
begin
  UpdateUI;
end;

procedure TFormOptions.CheckBoxUseProxyClick(Sender: TObject);
begin
  UpdateUI;
end;

procedure TFormOptions.ComboBoxURLShortenerSiteChange(Sender: TObject);
begin
  UpdateUI;
end;

procedure TFormOptions.FormCreate(Sender: TObject);
begin
  FSpeechConfigs := TCollection.Create(TSpeechConfig);
  FCaptionFontText := TCaptionFont.Create;
  FCaptionFontUserName := TCaptionFont.Create;
  FCaptionFontDateTime := TCaptionFont.Create;
end;

procedure TFormOptions.FormDestroy(Sender: TObject);
begin
  FCaptionFontDateTime.Free;
  FCaptionFontUserName.Free;
  FCaptionFontText.Free;
  FSpeechConfigs.Free;
end;

procedure TFormOptions.GetFromUserConfig;
var
  index: Integer;
begin
  case UserConfig.URLShortenerConfig.URLShortenerSite of
    usIsGd:
      index := 2;
    usBitLy:
      index := 1;
    usTinyURL:
      index := 0;
  end;
  ComboBoxURLShortenerSite.ItemIndex := index;
  UpDownAutoUpdateSecHome.Position := UserConfig.AutoUpdateSec.HomeTimeline;
  UpDownAutoUpdateSecUser.Position := UserConfig.AutoUpdateSec.UserTimeline;
  UpDownAutoUpdateSecReplies.Position := UserConfig.AutoUpdateSec.Replies;
  UpDownAutoUpdateSecDirectMessages.Position := UserConfig.AutoUpdateSec.DirectMessages;
  UpDownAutoUpdateSecFavorites.Position := UserConfig.AutoUpdateSec.Favorites;
  UpDownAutoUpdateSecPublicList.Position := UserConfig.AutoUpdateSec.PublicList;
  UpDownAutoUpdateSecPrivateList.Position := UserConfig.AutoUpdateSec.PrivateList;
  UpDownAutoUpdateSecSearch.Position := UserConfig.AutoUpdateSec.Search;
  EditProxyHost.Text := UserConfig.ProxyConfig.Host;
  UpDownProxyPort.Position := UserConfig.ProxyConfig.Port;
  CheckBoxUseProxy.Checked := UserConfig.ProxyConfig.UseProxy;
  UpDownListViewFontSize.Position := UserConfig.ListViewFont.FontSize;
  ComboBoxListViewFontName.Text := UserConfig.ListViewFont.FontName;
  ComboBoxListViewFontName.Items := Screen.Fonts;
  UpDownRequestStatusesCount.Position := UserConfig.RequestStatusesCount;

  FSpeechConfigs.Assign(UserConfig.SpeechConfigs);
  FCaptionFontText.Assign(UserConfig.CaptionFont);
  FCaptionFontUserName.Assign(UserConfig.CaptionFontUserName);
  FCaptionFontDateTime.Assign(UserConfig.CaptionFontDateTime);
  EditURLShortenerUserName.Text := UserConfig.URLShortenerConfig.UserName;
  EditURLShortenerPassword.Text := UserConfig.URLShortenerConfig.Password;
  EditURLShortenerAPIKey.Text := UserConfig.URLShortenerConfig.APIKey;
  CheckBoxComplementScreenName.Checked := UserConfig.ComplementScreenName;
  CheckBoxPlaySoundBeforeSpeech.Checked := UserConfig.PlaySoundConfig.PlaySound;
  EditSoundFileName.Text := UserConfig.PlaySoundConfig.soundFileName;
  CheckBoxSynchPlaySound.Checked := UserConfig.PlaySoundConfig.SynchPlaySound;
  UpdateUI;
  UpdateListViewSpeeches;
end;

procedure TFormOptions.LinkLabelAPIKeyClick(Sender: TObject);
begin
  OpenWebAsBrowser('http://bit.ly/account/your_api_key');
end;

procedure TFormOptions.ListViewSpeechesSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  UpdateUI;
end;

procedure TFormOptions.SetToUserConfig;
var
  site: TURLShortenerSite;
begin
  case ComboBoxURLShortenerSite.ItemIndex of
    2:
      site := usIsGd;
    1:
      site := usBitLy;
    0:
      site := usTinyURL;
  end;
  UserConfig.URLShortenerConfig.URLShortenerSite := site;
  UserConfig.PlaySoundConfig.PlaySound := CheckBoxPlaySoundBeforeSpeech.Checked;
  UserConfig.PlaySoundConfig.soundFileName := EditSoundFileName.Text;
  UserConfig.PlaySoundConfig.SynchPlaySound := CheckBoxSynchPlaySound.Checked;
  UserConfig.ComplementScreenName := CheckBoxComplementScreenName.Checked;
  UserConfig.URLShortenerConfig.UserName := EditURLShortenerUserName.Text;
  UserConfig.URLShortenerConfig.Password := EditURLShortenerPassword.Text;
  UserConfig.URLShortenerConfig.APIKey := EditURLShortenerAPIKey.Text;
  UserConfig.CaptionFont.Assign(FCaptionFontText);
  UserConfig.CaptionFontUserName.Assign(FCaptionFontUserName);
  UserConfig.CaptionFontDateTime.Assign(FCaptionFontDateTime);
  UserConfig.SpeechConfigs.Assign(FSpeechConfigs);
  UserConfig.ListViewFont.FontSize := UpDownListViewFontSize.Position;
  UserConfig.ListViewFont.FontName := ComboBoxListViewFontName.Text;
  UserConfig.ProxyConfig.Host := EditProxyHost.Text;
  UserConfig.ProxyConfig.Port := UpDownProxyPort.Position;
  UserConfig.ProxyConfig.UseProxy := CheckBoxUseProxy.Checked;
  UserConfig.AutoUpdateSec.HomeTimeline := UpDownAutoUpdateSecHome.Position;
  UserConfig.AutoUpdateSec.UserTimeline := UpDownAutoUpdateSecUser.Position;
  UserConfig.AutoUpdateSec.Replies := UpDownAutoUpdateSecReplies.Position;
  UserConfig.AutoUpdateSec.DirectMessages := UpDownAutoUpdateSecDirectMessages.Position;
  UserConfig.AutoUpdateSec.Favorites := UpDownAutoUpdateSecFavorites.Position;
  UserConfig.AutoUpdateSec.PublicList := UpDownAutoUpdateSecPublicList.Position;
  UserConfig.AutoUpdateSec.PrivateList := UpDownAutoUpdateSecPrivateList.Position;
  UserConfig.AutoUpdateSec.Search := UpDownAutoUpdateSecSearch.Position;
  UserConfig.RequestStatusesCount := UpDownRequestStatusesCount.Position;
  FormCaption.UpdateCaption;
end;

procedure TFormOptions.UpdateListViewSpeeches;
var
  i: Integer;
  speechConfig: TSpeechConfig;
  Item: TListItem;
begin
  ListViewSpeeches.Clear;

  for i := 0 to FSpeechConfigs.Count - 1 do
  begin
    speechConfig := FSpeechConfigs.Items[i] as TSpeechConfig;
    // ListViewSpeeches.AddItem(speechConfig.Lang, speechConfig);
    Item := ListViewSpeeches.Items.Add;
    Item.Caption := speechConfig.Lang;
    Item.SubItems.Add(speechConfig.VoiceName);
    Item.SubItems.Add(IntToStr(speechConfig.Volume));
    Item.SubItems.Add(IntToStr(speechConfig.Rate));
    Item.SubItems.Add(IntToStr(speechConfig.WaitingTimeSec));
    Item.Data := speechConfig;
  end;
end;

procedure TFormOptions.UpdateUI;
begin
  // PanelProxy.Enabled := CheckBoxUseProxy.Checked;
  Utils.EnableControlAll(PanelProxy, CheckBoxUseProxy.Checked);
  ButtonEditSpeechConfig.Enabled := ListViewSpeeches.Selected <> nil;
  Utils.EnableControlAll(PanelPlaySound, CheckBoxPlaySoundBeforeSpeech.Checked);
  Utils.EnableControlAll(PanelURLShorten, ComboBoxURLShortenerSite.ItemIndex = 1);
end;

end.
