unit tvFormUsers;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, VirtualTrees, ComCtrls, ToolWin, Menus, tvUserConfig, tvXMLBase, tvXMLUserManager, utFunctions,
  tvImageDownloader, tvCommunicator, Generics.Collections, ActnList, DateUtils;

const
  MIN_NODE_HEIGHT = 48;
  NODE_LEFT_MARGIN = 4;
  NODE_TOP_MARGIN = 2;

type
  TFormUsers = class(TForm)
    MainMenu: TMainMenu;
    StatusBar: TStatusBar;
    ToolBar1: TToolBar;
    TabControl: TTabControl;
    VirtualTreeView: TVirtualDrawTree;
    ActionList: TActionList;
    ActionOpenWebTwitterHome: TAction;
    ActionOpenWebHP: TAction;
    PopupMenuTreeView: TPopupMenu;
    popMenuItemWeb: TMenuItem;
    popMenuItemOpenWebHP: TMenuItem;
    popMenuItemOpenWebTwitterHome: TMenuItem;
    ActionShowFormUserOptions: TAction;
    popMenuItemUser: TMenuItem;
    popMenuItemShowFormUserOptions: TMenuItem;
    ActionRegetProfileImage: TAction;
    popMenuItemRegetProfileImage: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure VirtualTreeViewMeasureItem(Sender: TBaseVirtualTree; TargetCanvas: TCanvas; Node: PVirtualNode;
      var NodeHeight: Integer);
    procedure VirtualTreeViewDrawNode(Sender: TBaseVirtualTree; const PaintInfo: TVTPaintInfo);
    procedure FormDestroy(Sender: TObject);
    procedure VirtualTreeViewColumnResize(Sender: TVTHeader; Column: TColumnIndex);
    procedure VirtualDrawTreeScroll(Sender: TBaseVirtualTree; DeltaX, DeltaY: Integer);
    //procedure VirtualTreeViewHeaderClick(Sender: TVTHeader; HitInfo: TVTHeaderHitInfo);
    procedure VirtualTreeViewCompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode; Column: TColumnIndex;
      var Result: Integer);
    procedure ActionOpenWebTwitterHomeExecute(Sender: TObject);
    procedure ActionOpenWebHPExecute(Sender: TObject);
    procedure ActionShowFormUserOptionsExecute(Sender: TObject);
    procedure ActionRegetProfileImageExecute(Sender: TObject);
    procedure VirtualTreeViewHeaderClick(Sender: TVTHeader; Column: TColumnIndex; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private 錾 }
    FFormClosing: Boolean;
    FAddedUsers: TDictionary<string, TXMLUser>;
    FUpdatedVirtualTreeView: Boolean;
    procedure SetWindowRectToUserConfig;
    procedure UpdateVirtualTreeView;
    procedure AddChildNode(AUser: TXMLUser);
  public
    { Public 錾 }

    function GetSelectedNode: PVirtualNode;
    procedure SetBitmaps(AUrlBitmaps: TList<TUrlBitmap>);
    procedure SetCommunicatorResult(ARequest: TCommRequest);

  end;

  TTreeData = record
    User: TXMLUser;
    pNodeData: Pointer;
  end;

  PTreeData = ^TTreeData;

var
  FormUsers: TFormUsers;

implementation

uses tvFormMain, tvFormUserOptions, tvDBManager;
{$R *.dfm}

procedure TFormUsers.ActionOpenWebHPExecute(Sender: TObject);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
begin
  Node := GetSelectedNode;
  if Assigned(Node) then
  begin
    dataPtr := VirtualTreeView.GetNodeData(Node);
    dataPtr.User.OpenWebHP;
  end;
end;

procedure TFormUsers.ActionOpenWebTwitterHomeExecute(Sender: TObject);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
begin
  Node := GetSelectedNode;
  if Assigned(Node) then
  begin
    dataPtr := VirtualTreeView.GetNodeData(Node);
    dataPtr.User.OpenWebTwitterHome;
  end;
end;

procedure TFormUsers.ActionRegetProfileImageExecute(Sender: TObject);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
  User: TXMLUser;
  urlBitmap: TUrlBitmap;
begin
  Node := GetSelectedNode;
  if Assigned(Node) then
  begin
    dataPtr := VirtualTreeView.GetNodeData(Node);
    User := dataPtr.User;
    // User.ProfileImage.Free;
    // User.ProfileImage := nil;
    urlBitmap := TUrlBitmap.Create;
    urlBitmap.User := User;
    FormMainTweetVoice.ImageDownloader.EnqueueUrl(urlBitmap);
  end;
end;

procedure TFormUsers.ActionShowFormUserOptionsExecute(Sender: TObject);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
  User: TXMLUser;
begin
  Node := GetSelectedNode;
  if Assigned(Node) then
  begin
    dataPtr := VirtualTreeView.GetNodeData(Node);
    User := dataPtr.User;
    FormUserOptions.SetFrom(User);
    if (FormUserOptions.ShowModal = mrOK) then
    begin
      FormUserOptions.SetTo(User);
      Screen.Cursor := crHourGlass;
      try
        DBManager.InsertOrUpdateUser(User);
        VirtualTreeView.RepaintNode(Node);
      finally
        Screen.Cursor := crDefault;
      end;
    end;
  end;

end;

procedure TFormUsers.AddChildNode(AUser: TXMLUser);
var
  Node: PVirtualNode;
  dataPtr: PTreeData;
  urlBitmap: TUrlBitmap;
  screenName: string;
begin
  screenName := LowerCase(AUser.screenName);
  if Not FAddedUsers.ContainsKey(screenName) then
  begin
    Node := VirtualTreeView.AddChild(nil);
    dataPtr := VirtualTreeView.GetNodeData(Node);
    dataPtr.User := AUser;
    // 摜_E[hv
    if (Not Assigned(AUser.ProfileImage) or AUser.ProfileImageUrlChanged) and (Not AUser.ProfileImageRequested) and
      (AUser.ID <> 0) then
    begin
      urlBitmap := TUrlBitmap.Create;
      urlBitmap.User := AUser;
      FormMainTweetVoice.ImageDownloader.EnqueueUrl(urlBitmap);
      AUser.ProfileImageRequested := true;
      AUser.ProfileImageUrlChanged := true;
    end;
    FAddedUsers.Add(screenName, AUser);
  end;
end;

procedure TFormUsers.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetWindowRectToUserConfig;
end;

procedure TFormUsers.FormCreate(Sender: TObject);
begin
  FAddedUsers := TDictionary<string, TXMLUser>.Create;
  VirtualTreeView.NodeDataSize := Sizeof(TTreeData);
end;

procedure TFormUsers.FormDestroy(Sender: TObject);
begin
  FFormClosing := true;
  FAddedUsers.Free;
end;

procedure TFormUsers.FormShow(Sender: TObject);
begin
  if (UserConfig.IsLoaded) then
  begin
    Left := UserConfig.FormUsersRect.Left;
    Top := UserConfig.FormUsersRect.Top;
    Width := UserConfig.FormUsersRect.Width;
    Height := UserConfig.FormUsersRect.Height;
  end
  else
  begin
    SetWindowRectToUserConfig;
  end;
end;

function TFormUsers.GetSelectedNode: PVirtualNode;
var
  Node: PVirtualNode;
begin
  Result := nil;
  Node := VirtualTreeView.RootNode.FirstChild;
  while (Node <> nil) do
  begin
    if (VirtualTreeView.Selected[Node]) then
    begin
      Result := Node;
      break;
    end;
    Node := Node.NextSibling;
  end;
end;

procedure TFormUsers.SetBitmaps(AUrlBitmaps: TList<TUrlBitmap>);
var
  urlBitmap: TUrlBitmap;
  Node: PVirtualNode;
  dataPtr: PTreeData;
begin
  if (FFormClosing) then
    Exit;

  // Ə\
  for urlBitmap in AUrlBitmaps do
  begin
    Node := VirtualTreeView.RootNode.FirstChild;
    while (Node <> nil) do
    begin
      dataPtr := VirtualTreeView.GetNodeData(Node);
      if Not Assigned(dataPtr.User.ProfileImage) and (urlBitmap.User = dataPtr.User) then
      begin
        dataPtr.User.ProfileImage.Free;
        dataPtr.User.ProfileImage := urlBitmap.bitmap;
        if Not Assigned(urlBitmap.bitmap) then
        begin
          // C[W擾łȂ̂Ŏ̂ƂĎs
          dataPtr.User.ProfileImageRequested := false;
        end;
        urlBitmap.bitmap := nil;
        break;
      end;
      Node := Node.NextSibling;
    end;

    urlBitmap.Free;
  end;

  VirtualTreeView.Refresh;
end;

procedure TFormUsers.SetCommunicatorResult(ARequest: TCommRequest);
var
  // User: TXMLUser;
  Status: TXMLStatus;
begin
  VirtualTreeView.BeginUpdate;
  if Not FUpdatedVirtualTreeView then
  begin
    UpdateVirtualTreeView;
    FUpdatedVirtualTreeView := true;
  end;

  // if (ARequest is TCommRequestStatuses) then
  begin
    for Status in ARequest.Statuses do
    begin
      AddChildNode(Status.User);
      if Assigned(Status.RetweetedStatus) then
      begin
        AddChildNode(Status.RetweetedStatus.User);
      end;
    end;
    if Assigned(ARequest.User) then
    begin
      AddChildNode(ARequest.User);
    end;
  end;
  VirtualTreeView.EndUpdate;
end;

procedure TFormUsers.SetWindowRectToUserConfig;
begin
  UserConfig.FormUsersRect.Left := Left;
  UserConfig.FormUsersRect.Top := Top;
  UserConfig.FormUsersRect.Width := Width;
  UserConfig.FormUsersRect.Height := Height;

end;

procedure TFormUsers.UpdateVirtualTreeView;
var
  User: TXMLUser;
begin
  for User in XMLUserManager.NameUsers.Values do
  begin
    AddChildNode(User);
  end;
end;

procedure TFormUsers.VirtualTreeViewColumnResize(Sender: TVTHeader; Column: TColumnIndex);
begin
  VirtualTreeView.ReinitChildren(nil, false);
end;

procedure TFormUsers.VirtualTreeViewCompareNodes(Sender: TBaseVirtualTree; Node1, Node2: PVirtualNode;
  Column: TColumnIndex; var Result: Integer);
var
  data1Ptr: PTreeData;
  data2Ptr: PTreeData;
  user1: TXMLUser;
  user2: TXMLUser;
begin
  data1Ptr := VirtualTreeView.GetNodeData(Node1);
  data2Ptr := VirtualTreeView.GetNodeData(Node2);
  user1 := data1Ptr.User;
  user2 := data2Ptr.User;

  case Column of
    - 1:
      begin
        if (user1.ID = user2.ID) then
        begin
          Result := 0;
        end
        else if (user1.ID > user2.ID) then
        begin
          Result := 1;
        end
        else
        begin
          Result := -1;
        end;
      end;
    0:
      Result := Utils.CompareString(user1.ProfileImageUrl, user2.ProfileImageUrl);
    1:
      Result := Utils.CompareString(user1.Name, user2.Name);
    2:
      Result := Utils.CompareString(user1.NamePronounciation, user2.NamePronounciation);
    3:
      Result := Utils.CompareString(user1.url, user2.url);
    4:
      Result := Utils.CompareString(user1.Location, user2.Location);
    5:
      Result := Utils.CompareString(user1.Description, user2.Description);
    6:
      Result := user1.FriendsCount - user2.FriendsCount;
    7:
      Result := user1.FollowersCount - user2.FollowersCount;
    8:
      Result := user1.StatusesCount - user2.StatusesCount;
    9:
      Result := user1.FavouritesCount - user2.FavouritesCount;
    10:
      Result := Utils.CompareString(user1.Status.Text, user2.Status.Text);
    11:
      Result := DateUtils.CompareDateTime(user1.CreatedAt, user2.CreatedAt);
  end;

end;

procedure TFormUsers.VirtualTreeViewDrawNode(Sender: TBaseVirtualTree; const PaintInfo: TVTPaintInfo);
var
  TargetCanvas: TCanvas;
  Node: PVirtualNode;
  Column: TColumnIndex;
  CellRect: TRect;
  dataPtr: PTreeData;

  s: WideString;
  rect: TRect;
  ForeColor: TColor;
  BackColor: TColor;
  textFlags: Cardinal;
begin
  TargetCanvas := PaintInfo.Canvas;
  Node := PaintInfo.Node;
  Column := PaintInfo.Column;
  CellRect := PaintInfo.CellRect;

  dataPtr := VirtualTreeView.GetNodeData(Node);

  rect := CellRect;

  if (VirtualTreeView.Selected[Node]) then
  begin
    ForeColor := Utils.NotColor(dataPtr.User.GetProfileTextColor);
    BackColor := Utils.NotColor(dataPtr.User.GetProfileSidebarFillColor);
  end
  else
  begin
    ForeColor := dataPtr.User.GetProfileTextColor;
    BackColor := dataPtr.User.GetProfileSidebarFillColor;
  end;

  TargetCanvas.Font.Name := UserConfig.ListViewFont.FontName;
  TargetCanvas.Font.Size := UserConfig.ListViewFont.FontSize;
  TargetCanvas.Font.Color := ForeColor;
  TargetCanvas.Brush.Color := BackColor;
  TargetCanvas.FillRect(rect);
  textFlags := DT_WORDBREAK or DT_NOPREFIX or DT_END_ELLIPSIS;
  if (Column = 0) then
  begin
    if Assigned(dataPtr.User.ProfileImage) then
    begin
      TargetCanvas.Draw(rect.Left, rect.Top, dataPtr.User.ProfileImage);
    end;
  end
  else
  begin
    case Column of
      1:
        s := dataPtr.User.Name + #$d#$a + '@' + dataPtr.User.screenName;
      2:
        s := dataPtr.User.NamePronounciation;
      3:
        s := dataPtr.User.url;
      4:
        s := dataPtr.User.Location;
      5:
        s := dataPtr.User.Description;
      6:
        begin
          s := IntToStr(dataPtr.User.FriendsCount);
          textFlags := DT_RIGHT;
        end;
      7:
        begin
          s := IntToStr(dataPtr.User.FollowersCount);
          textFlags := DT_RIGHT;
        end;
      8:
        begin
          s := IntToStr(dataPtr.User.StatusesCount);
          textFlags := DT_RIGHT;
        end;
      9:
        begin
          s := IntToStr(dataPtr.User.FavouritesCount);
          textFlags := DT_RIGHT;
        end;
      10:
        s := dataPtr.User.Status.Text;
      11:
        s := FormatDateTime('yyyy/mm/dd hh:nn:ss', dataPtr.User.LocalTimeCreatedAt);
    end;

    if (Column = 1) then
    begin
      TargetCanvas.Font.Style := [fsBold];
    end
    else
    begin
      TargetCanvas.Font.Style := [];
    end;

    rect.Top := rect.Top + NODE_TOP_MARGIN;
    rect.Left := rect.Left + NODE_LEFT_MARGIN;
    Windows.DrawTextW(TargetCanvas.Handle, PWideChar(s), -1, rect, textFlags);
  end;

  if (VirtualTreeView.Selected[Node]) then
  begin
    // ㉺ɐԂ
    TargetCanvas.Pen.Width := 2;
    TargetCanvas.Pen.Color := clRed;
    TargetCanvas.MoveTo(CellRect.Left, CellRect.Top + 1);
    TargetCanvas.LineTo(CellRect.Right, CellRect.Top + 1);
    TargetCanvas.MoveTo(CellRect.Left, CellRect.Bottom - 1);
    TargetCanvas.LineTo(CellRect.Right, CellRect.Bottom - 1);

  end;

end;

procedure TFormUsers.VirtualTreeViewHeaderClick(Sender: TVTHeader; Column: TColumnIndex; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Column >= 0) then
  begin
    if (VirtualTreeView.Header.SortColumn = Column) then
    begin
      if (VirtualTreeView.Header.SortDirection = sdDescending) then
      begin
        VirtualTreeView.Header.SortDirection := sdAscending;
      end
      else
      begin
        VirtualTreeView.Header.SortColumn := -1;
        VirtualTreeView.Header.SortDirection := sdDescending;
      end;
    end
    else
    begin
      VirtualTreeView.Header.SortColumn := Column;
      VirtualTreeView.Header.SortDirection := sdDescending;
    end;
  end;
end;

procedure TFormUsers.VirtualTreeViewMeasureItem(Sender: TBaseVirtualTree; TargetCanvas: TCanvas; Node: PVirtualNode;
  var NodeHeight: Integer);
var
  dataPtr: PTreeData;
  columnW: Integer;
  s: WideString;
  rect: TRect;
begin
  dataPtr := VirtualTreeView.GetNodeData(Node);
  columnW := VirtualTreeView.Header.Columns[10].Width;

  TargetCanvas.Font.Name := UserConfig.ListViewFont.FontName;
  TargetCanvas.Font.Size := UserConfig.ListViewFont.FontSize;

  s := dataPtr.User.Status.Text;
  rect.Left := NODE_LEFT_MARGIN;
  rect.Top := NODE_TOP_MARGIN;
  rect.Right := columnW;
  rect.Bottom := 10000;
  Windows.DrawTextW(TargetCanvas.Handle, PWideChar(s), -1, rect, DT_WORDBREAK or DT_CALCRECT);

  if (rect.Bottom > MIN_NODE_HEIGHT) then
  begin
    NodeHeight := rect.Bottom;
  end
  else
  begin
    NodeHeight := MIN_NODE_HEIGHT;
  end;

  s := dataPtr.User.Description;
  rect.Right := VirtualTreeView.Header.Columns[5].Width;

  Windows.DrawTextW(TargetCanvas.Handle, PWideChar(s), -1, rect, DT_WORDBREAK or DT_CALCRECT);

  if (rect.Bottom > NodeHeight) then
  begin
    NodeHeight := rect.Bottom;
  end
  else
  begin
    NodeHeight := MIN_NODE_HEIGHT;
  end;

end;

procedure TFormUsers.VirtualDrawTreeScroll(Sender: TBaseVirtualTree; DeltaX, DeltaY: Integer);
begin
  // VirtualTreeView.Repaint;
end;

end.
