unit tvJSON;

interface

uses
  Windows, Generics.Collections, Graphics, ExtCtrls, SysUtils, utFunctions,
  tvUserConfig, tvXMLBase, Generics.Defaults, Forms, DateUtils, Classes, superobject;

type
  TJSONParser = class
  private
    FStatuses: TList<TXMLStatus>;
    FXMLParser: TXMLParser;

    // FJSONObject: TlkJSONobject;
  public
    constructor Create(AXMLParser: TXMLParser);
    destructor Destroy; override;

    procedure SetFromJSON(AStream: TStream);

    property Statuses: TList<TXMLStatus>read FStatuses;
  end;

implementation

uses tvXMLUserManager, tvXMLStatusManager;

{ TJSONParser }

constructor TJSONParser.Create(AXMLParser: TXMLParser);
begin
  // FJSONObject := TlkJSONobject.Create;
  FXMLParser := AXMLParser;
  FStatuses := TList<TXMLStatus>.Create;
end;

destructor TJSONParser.Destroy;
begin
  FStatuses.Free;
  // FJSONObject.Free;
  inherited;
end;

procedure TJSONParser.SetFromJSON(AStream: TStream);
var
  superobject: ISuperObject;
  superArray: TSuperArray;
  // JSONNumber: TlkJSONnumber;
  // JSONString: TlkJSONstring;
  child: ISuperObject;
  i: Integer;
  status: TXMLStatus;
  dateTime: TDateTime;
  dateTimeStr: string;
  userName: string;
  profileImageUrl: string;
  user: TXMLUser;
  userID: Int64;
  utf8Text: WideString;
  startTime: TDateTime;
  elapsedTime: Int64;
begin
  userID := 0;
  FStatuses.Clear;

  startTime := Now;
  superobject := TSuperObject.ParseStream(AStream, false);

  elapsedTime := DateUtils.MilliSecondsBetween(Now, startTime);
  Utils.OutputDebugString(Format('ParseString %d msec.', [elapsedTime]));
  startTime := Now;
  try
    superArray := superobject.A['results'];

    // JSONList := JSONObject.Field['results'] as TlkJSONlist;
    for i := 0 to superArray.Length - 1 do
    begin
      child := superArray.O[i];
      status := TXMLStatus.Create(FXMLParser, false);

      status.ID := child.i['id'];
      userID := child.i['from_user_id'];
      userName := child.S['from_user'];

      utf8Text := child.S['text'];
      status.Text := Utils.HTMLDecode(utf8Text);

      profileImageUrl := child.S['profile_image_url'];

      FXMLParser.GetSource(status.Source, Utils.HTMLDecode(child.S['source']));

      dateTimeStr := child.S['created_at'];
      if (FXMLParser.GetDateTime(dateTimeStr, dateTime)) then
      begin
        status.CreatedAt := dateTime;
      end;

      user := XMLUserManager.GetUser(userName);

      if Assigned(user) then
      begin
        status.user := user;
      end
      else
      begin
        user := TXMLUser.Create(FXMLParser);
        user.ID := userID;
        user.ScreenName := userName;
        user.profileImageUrl := profileImageUrl;
        status.user := XMLUserManager.RegisterUser(user);
      end;

      status := XMLStatusManager.RegisterStatus(status);
      FStatuses.Add(status);
    end;
    elapsedTime := DateUtils.MilliSecondsBetween(Now, startTime);
    Utils.OutputDebugString(Format('superArray %d msec.', [elapsedTime]));

  finally
    // superobject.Free;
  end;

end;

end.
