unit tvXMLBase;

interface

uses
  Windows, Generics.Collections, Graphics, ExtCtrls, XMLDoc, XMLIntf, SysUtils, utFunctions,
  tvUserConfig, tvRectView, tvXMLResponse, Clipbrd, PerlRegEx, StrUtils;

type
  TXMLParser = class;
  TXMLStatus = class;
  TXMLUser = class;
  // TXMLSearch = class;
  TAPIType = (atDefault, atHomeTimeLine, atDirectMessages, atURLShortener);

  TXMLHash = class
  public
    HourlyLimit: Integer;
    RemainingHits: Integer;
    ResetTimeInSeconds: Int64;
    ResetTime: string;
    LocalResetTime: TDateTime;
    constructor Create(AParent: TXMLParser);
    destructor Destroy; override;

    procedure SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure CopyFrom(AHash: TXMLHash);
    // function LocalResetTime: TDateTime;

  private
    FParent: TXMLParser;
  end;

  TXMLSearch = class
  public
    ID: Int64;
    Name: string;
    Query: string;
    CreatedAt: TDateTime;

    constructor Create(AParent: TXMLParser);
    destructor Destroy; override;

    procedure SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure CopyFrom(ASearch: TXMLSearch);

  private
    FParent: TXMLParser;
  end;

  TXMLList = class
  public
    ID: Int64;
    Name: string;
    FullName: string;
    Description: string;
    SubscriberCount: Integer;
    MemberCount: Integer;
    Uri: string;
    Mode: string;
    User: TXMLUser;
    procedure SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    constructor Create(AParent: TXMLParser);
    destructor Destroy; override;
  private
    FParent: TXMLParser;
  end;

  TXMLUser = class
  private
    FProfileTextColor: TColor;
    FProfileSidebarFillColor: TColor;
    FProfileSidebarFillColor2: TColor;
  public
    ID: Int64;
    Name: string;
    ScreenName: string;
    NamePronounciation: string;
    Location: string;
    Description: string;
    ProfileImageUrl: string;
    ProfileImage: TBitmap;
    ProfileImageRequested: boolean;
    ProfileImageUrlChanged: boolean;
    Url: string;
    IsProtected: boolean;
    FollowersCount: Integer;
    ProfileBackgroundColor: TColor;
    ProfileBackgroundImageUrl: string;
    ProfileBackgroundImage: TBitmap;
    ProfileBackgroundTile: boolean;
    ProfileTextColor: TColor;
    ProfileLinkColor: TColor;
    ProfileSidebarFillColor: TColor;
    ProfileSidebarBorderColor: TColor;
    FriendsCount: Integer;
    CreatedAt: TDateTime;
    FavouritesCount: Integer;
    UtcOffset: Integer;
    TimeZone: string;
    Notifications: boolean;
    GeoEnabled: boolean;
    Verified: boolean;
    Following: boolean;
    Lang: string;
    StatusesCount: Integer;
    IsContributorsEnabled: boolean;
    Status: TXMLStatus;
    Lists: TObjectList<TXMLList>;
    procedure SetFromXML(AXMLNode: IXMLNode);
    procedure CopyFrom(AUser: TXMLUser);
    procedure UpdateStatus(AStatus: TXMLStatus);
    procedure OpenWebTwitterHome;
    procedure OpenWebHP;
    function GetNamePronounciation(ALang: string): string;
    function GetNameEnclosedInParentheses: string;
    function LocalTimeCreatedAt: TDateTime;
    function CreateUserRectView(AInsertBreak: boolean = true; ARetweetStatus: TXMLStatus = nil): TRectView;

    function IsFollowingMe: boolean;
    function IsFollowedByMe: boolean;
    function FollowingIndex: Integer;

    procedure AdjustColors;
    function GetProfileTextColor: TColor;
    function GetProfileSidebarFillColor: TColor;
    function GetProfileSidebarFillColor2: TColor;
    function LowScreenName: string;
    constructor Create(AParent: TXMLParser);
    destructor Destroy; override;

  private
    FParent: TXMLParser;
  end;

  TXMLSource = record
    Url: string;
    Name: String;
  end;

  TXMLStatus = class
  public
    CreatedAt: TDateTime;
    ID: Int64;
    Text: string;
    Source: TXMLSource;
    Truncated: boolean;
    Favorited: boolean;
    InReplyToStatusID: Int64;
    InReplyToUserID: Int64;
    InReplyToScreenName: string;
    IsRead: boolean;
    User: TXMLUser;
    RecipientUser: TXMLUser;
    RetweetedStatusID: Int64;
    RetweetedStatus: TXMLStatus;
    constructor Create(AParent: TXMLParser; AUserEnabled: boolean);
    destructor Destroy; override;
    procedure SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure CopyFrom(AStatus: TXMLStatus);
    function LocalTimeCreatedAt: TDateTime;
    function CreateSourceRectView: TRectView;
    procedure AddSourceToRectView(ARectView: TRectView);
    procedure OpenWebTweet;
    function GetRecipientUserID: Integer;

    function GetTweetUrl: string;
    function GetAllText: string;
    function GetRead: boolean;
    function GetRetweetedStatus: TXMLStatus;
    function ContainsText(AText: string): boolean;
    function GetIconForSorting: Integer;
  private
    FParent: TXMLParser;
    FUserEnabled: boolean;
  end;

  TXMLParser = class
  public
    constructor Create;
    destructor Destroy; override;
    procedure SetFromXML(AXMLDoc: IXMLDocument; AAPIType: TAPIType = atDefault);
    procedure SetStatusFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetStatusesFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetDMsFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetUserFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetUsersFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetListsListFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetIDsFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetSearchesFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure SetSearchFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
    procedure GetSource(var ASource: TXMLSource; AText: string);
    function GetDateTime(formatDateStr: string; var ADateTime: TDateTime): boolean;
    function ConvertToLocalTime(ADateTime: TDateTime): TDateTime;
  private
    FStatus: TXMLStatus;
    FStatuses: TList<TXMLStatus>;
    FIDs: TList<Int64>;
    FLists: TList<TXMLList>;
    FSearch: TXMLSearch;
    FSearches: TList<TXMLSearch>;
    FRegExpHref: TPerlRegEx;
    FRegExpDateTime: TPerlRegEx;
    FRegExpDateTimeJSON: TPerlRegEx;
    FTzInfo: TIME_ZONE_INFORMATION;
    FUser: TXMLUser;
    FUsers: TList<TXMLUser>;
    FHash: TXMLHash;
    FResponse: TXMLResponse;
  public
    property Lists: TList<TXMLList>read FLists;
    property Searches: TList<TXMLSearch>read FSearches;
    property Search: TXMLSearch read FSearch write FSearch;
    property Hash: TXMLHash read FHash;
    property Status: TXMLStatus read FStatus;
    property User: TXMLUser read FUser write FUser;
    property Statuses: TList<TXMLStatus>read FStatuses;
    property Users: TList<TXMLUser>read FUsers;
    property IDs: TList<Int64>read FIDs;
    property Response: TXMLResponse read FResponse;
  end;

function StrToColor(AHexString: string): TColor;

implementation

uses
  Dialogs, Variants, tvXMLUserManager, tvXMLStatusManager, DateUtils;

function StrToColor(AHexString: string): TColor;
var
  // c: Integer;
  r, g, b: Integer;
begin
  Result := clBlack;
  if (Length(AHexString) >= 6) then
  begin
    r := StrToIntDef('$' + Copy(AHexString, 1, 2), 0);
    g := StrToIntDef('$' + Copy(AHexString, 3, 2), 0);
    b := StrToIntDef('$' + Copy(AHexString, 5, 2), 0);
    Result := RGB(r, g, b);
  end;
end;

{ TXMLBase }

function TXMLParser.ConvertToLocalTime(ADateTime: TDateTime): TDateTime;
var
  systemTime: TSystemTime;
begin
  // UTCRs[^̃[JԂɕϊB
  DateTimeToSystemTime(ADateTime, systemTime);
  SystemTimeToTzSpecificLocalTime(@FTzInfo, systemTime, systemTime);
  Result := SystemTimeToDateTime(systemTime);
end;

constructor TXMLParser.Create;
begin
  FSearches := TList<TXMLSearch>.Create;
  FLists := TList<TXMLList>.Create;
  FHash := TXMLHash.Create(Self);
  FStatuses := TList<TXMLStatus>.Create;
  FUsers := TList<TXMLUser>.Create;
  FIDs := TList<Int64>.Create;
  FResponse := TXMLResponse.Create;

  FRegExpHref := TPerlRegEx.Create(nil);
  FRegExpHref.Options := [preCaseLess];
  FRegExpHref.RegEx := '<a\s+href="([^"]+)".*>(.*)</a>';

  FRegExpDateTime := TPerlRegEx.Create(nil);
  FRegExpDateTime.Options := [preCaseLess];
  FRegExpDateTime.RegEx := '\w\w\w (\w\w\w) (\d\d) (\d\d:\d\d:\d\d) \+(\d\d)(\d\d) (\d\d\d\d)';

  FRegExpDateTimeJSON := TPerlRegEx.Create(nil);
  FRegExpDateTimeJSON.Options := [preCaseLess];
  FRegExpDateTimeJSON.RegEx := '\w\w\w,\s+(\d\d)\s+(\w\w\w)\s+(\d\d\d\d)\s+(\d\d:\d\d:\d\d)';

  // ^C][擾
  GetTimeZoneInformation(FTzInfo);
end;

destructor TXMLParser.Destroy;
begin
  FRegExpDateTimeJSON.Free;
  FRegExpDateTime.Free;
  FRegExpHref.Free;
  FResponse.Free;
  FIDs.Free;
  FUsers.Free;
  FStatuses.Free;
  FUser.Free;
  FSearch.Free;
  FHash.Free;
  FLists.Free;
  FSearches.Free;
  inherited;
end;

function TXMLParser.GetDateTime(formatDateStr: string; var ADateTime: TDateTime): boolean;
var
  dateStr: string;
begin
  Result := false;
  FRegExpDateTime.Subject := formatDateStr;
  if (FRegExpDateTime.Match) then
  begin
    dateStr := Format('%s %s %s %s', [FRegExpDateTime.SubExpressions[1], FRegExpDateTime.SubExpressions[2],
      FRegExpDateTime.SubExpressions[6], FRegExpDateTime.SubExpressions[3]]);
    try
      begin
        ADateTime := VarToDateTime(dateStr); // + StrToTime(tzStr);
        Result := true;
      end;
    except
      on E: Exception do
      begin
      end;
    end;
  end
  else
  begin
    FRegExpDateTimeJSON.Subject := formatDateStr;
    if (FRegExpDateTimeJSON.Match) then
    begin
      dateStr := Format('%s %s %s %s', [FRegExpDateTimeJSON.SubExpressions[2], FRegExpDateTimeJSON.SubExpressions[1],
        FRegExpDateTimeJSON.SubExpressions[3], FRegExpDateTimeJSON.SubExpressions[4]]);
      try
        begin
          ADateTime := VarToDateTime(dateStr); // + StrToTime(tzStr);
          Result := true;
        end;
      except
        on E: Exception do
        begin
        end;
      end;
    end;
  end;
end;

procedure TXMLParser.GetSource(var ASource: TXMLSource; AText: string);
begin
  ASource.Url := '';
  ASource.Name := AText;
  FRegExpHref.Subject := AText;
  if (FRegExpHref.Match) then
  begin
    ASource.Url := FRegExpHref.SubExpressions[1];
    ASource.Name := FRegExpHref.SubExpressions[2];
  end;
end;

procedure TXMLParser.SetDMsFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  i: Integer;
  Status: TXMLStatus;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'direct_message') then
    begin
      Status := TXMLStatus.Create(Self, true);
      Status.SetFromXML(childXMLNode, atDirectMessages);
      Status := XMLStatusManager.RegisterStatus(Status);

      FStatuses.Add(Status);
    end;
  end;
end;

procedure TXMLParser.SetFromXML(AXMLDoc: IXMLDocument; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  i: Integer;
  sText: string;
begin
  FUsers.Clear;
  FStatuses.Clear;
  FLists.Clear;
  FIDs.Clear;
  FSearches.Clear;
  FResponse.Clear;
  for i := 0 to AXMLDoc.Node.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLDoc.Node.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) then
    begin
      sText := childXMLNode.NodeName;
      if (sText = 'statuses') then
      begin
        SetStatusesFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'users') then
      begin
        SetUsersFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'user') then
      begin
        SetUserFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'status') then
      begin
        SetStatusFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'direct-messages') then
      begin
        SetDMsFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'lists_list') then
      begin
        SetListsListFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'saved_searches') then
      begin
        SetSearchesFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'saved_search') then
      begin
        SetSearchFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'id_list') then
      begin
        SetIDsFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'hash') then
      begin
        FHash.SetFromXML(childXMLNode, AAPIType);
      end
      else if (sText = 'response') then
      begin
        FResponse.SetFromXML(childXMLNode);
      end;
    end;
  end;
end;

procedure TXMLParser.SetIDsFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  ID: Int64;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'ids') then
    begin
      for j := 0 to childXMLNode.ChildNodes.Count - 1 do
      begin
        grandchildXMLNode := childXMLNode.ChildNodes[j];
        if (grandchildXMLNode.NodeType = ntElement) and (grandchildXMLNode.NodeName = 'id') then
        begin
          ID := StrToIntDef(grandchildXMLNode.Text, -1);
          if (ID > 0) then
          begin
            FIDs.Add(ID);
          end;
        end;
      end;
    end;
  end;
end;

procedure TXMLParser.SetListsListFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  list: TXMLList;
  _user: TXMLUser;
begin
  _user := nil;
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'lists') then
    begin
      for j := 0 to childXMLNode.ChildNodes.Count - 1 do
      begin
        grandchildXMLNode := childXMLNode.ChildNodes[j];
        if (grandchildXMLNode.NodeType = ntElement) and (grandchildXMLNode.NodeName = 'list') then
        begin
          list := TXMLList.Create(Self);
          list.SetFromXML(grandchildXMLNode, AAPIType);
          FLists.Add(list);
          _user := list.User;
        end;
      end;
    end;
  end;

  if Assigned(_user) then
  begin
    // ̂ƂAʂ̃[UƂƂ͂蓾Ȃ
    _user.Lists.Clear;
    for list in FLists do
    begin
      _user.Lists.Add(list);
    end;
  end;
end;

procedure TXMLParser.SetSearchesFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  i: Integer;
  Search: TXMLSearch;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'saved_search') then
    begin
      Search := TXMLSearch.Create(Self);
      Search.SetFromXML(childXMLNode, AAPIType);
      FSearches.Add(Search);
    end;
  end;
end;

procedure TXMLParser.SetSearchFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
begin
  FSearch.Free;
  FSearch := TXMLSearch.Create(Self);
  FSearch.SetFromXML(AXMLNode, AAPIType);
end;

procedure TXMLParser.SetStatusesFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  i: Integer;
  Status: TXMLStatus;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'status') then
    begin
      Status := TXMLStatus.Create(Self, true);
      Status.SetFromXML(childXMLNode, AAPIType);
      Status := XMLStatusManager.RegisterStatus(Status);

      FStatuses.Add(Status);
    end;
  end;
end;

procedure TXMLParser.SetStatusFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
begin
  FStatus := TXMLStatus.Create(Self, true);
  FStatus.SetFromXML(AXMLNode, AAPIType);
  FStatus := XMLStatusManager.RegisterStatus(FStatus);
end;

procedure TXMLParser.SetUserFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  i: Integer;
begin
  FUser.Free;
  FUser := TXMLUser.Create(Self);
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    FUser.SetFromXML(childXMLNode);
  end;

  // FUser := XMLUserManager.RegisterUser(FUser);
  FUser.AdjustColors;
end;

procedure TXMLParser.SetUsersFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode, grandChildNode: IXMLNode;
  i, j: Integer;
  _user: TXMLUser;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntElement) and (childXMLNode.NodeName = 'user') then
    begin
      _user := TXMLUser.Create(Self);
      for j := 0 to childXMLNode.ChildNodes.Count - 1 do
      begin
        grandChildNode := childXMLNode.ChildNodes[j];
        _user.SetFromXML(grandChildNode);
      end;

      _user := XMLUserManager.RegisterUser(_user);
      _user.AdjustColors;
      FUsers.Add(_user);
    end;
  end;
end;

{ TXMLStatus }

procedure TXMLStatus.AddSourceToRectView(ARectView: TRectView);
var
  UrlTextItem: TUrlTextItem;
  TextItem: TPlainTextItem;
  Url: string;
begin
  Url := Source.Url;
  if (Source.Name = 'web') then
  begin
    Url := 'http://twitter.com/';
  end;
  if (Url <> '') then
  begin
    UrlTextItem := TUrlTextItem.Create(ARectView);
    UrlTextItem.Text := Source.Name;
    UrlTextItem.Url := Url;
    ARectView.Items.Add(UrlTextItem);
  end
  else
  begin
    TextItem := TPlainTextItem.Create(ARectView);
    TextItem.Text := Source.Name;
    ARectView.Items.Add(TextItem);
  end;
end;

function TXMLStatus.ContainsText(AText: string): boolean;
begin
  Result := StrUtils.ContainsText(Text, AText) or StrUtils.ContainsText(User.Name, AText) or StrUtils.ContainsText
    (User.ScreenName, AText);
end;

procedure TXMLStatus.CopyFrom(AStatus: TXMLStatus);
begin
  Text := AStatus.Text;
  Source := AStatus.Source;
  CreatedAt := AStatus.CreatedAt;
  Truncated := AStatus.Truncated;
  InReplyToStatusID := AStatus.InReplyToStatusID;
  InReplyToUserID := AStatus.InReplyToUserID;
  InReplyToScreenName := AStatus.InReplyToScreenName;
  RetweetedStatusID := AStatus.RetweetedStatusID;
  RetweetedStatus := AStatus.RetweetedStatus;
end;

constructor TXMLStatus.Create(AParent: TXMLParser; AUserEnabled: boolean);
begin
  FParent := AParent;
  FUserEnabled := AUserEnabled;
  CreatedAt := 0;
end;

function TXMLStatus.CreateSourceRectView: TRectView;
var
  RectView: TRectView;
  UrlTextItem: TUrlTextItem;
  TextItem: TPlainTextItem;
  Url: string;

begin
  if Assigned(RecipientUser) then
  begin
    RectView := RecipientUser.CreateUserRectView();
  end
  else
  begin
    RectView := TRectView.Create;
    AddSourceToRectView(RectView);
  end;
  Result := RectView;
end;

destructor TXMLStatus.Destroy;
begin
  inherited;
end;

function TXMLStatus.GetAllText: string;
begin
  Result := '@' + User.ScreenName + #9 + User.Name + #9 + Text + #9 + FormatDateTime('yy/mm/dd hh:mm:ss',
    LocalTimeCreatedAt) + #9 + Source.Name;
end;

function TXMLStatus.GetIconForSorting: Integer;
begin
  Result := 0;
  if (InReplyToStatusID <> 0) then
  begin
    Result := 1;
  end
  else if (RetweetedStatusID > 0) then
  begin
    Result := 2;
  end;
end;

function TXMLStatus.GetRead: boolean;
var
  Status: TXMLStatus;
begin
  Status := Self;
  if Assigned(Status.RetweetedStatus) then
  begin
    Status := Status.RetweetedStatus;
  end;
  Result := Status.IsRead;
end;

function TXMLStatus.GetRecipientUserID: Integer;
begin
  Result := 0;
  if Assigned(RecipientUser) then
  begin
    Result := RecipientUser.ID;
  end;
end;

function TXMLStatus.GetRetweetedStatus: TXMLStatus;
begin
  Result := Self;
  if Assigned(Result.RetweetedStatus) then
  begin
    Result := Result.RetweetedStatus;
  end;
end;

function TXMLStatus.GetTweetUrl: string;
begin
  Result := TWITTER_BASE_URL + Self.User.ScreenName + '/status/' + IntToStr(Self.ID);
end;

function TXMLStatus.LocalTimeCreatedAt: TDateTime;
begin
  Result := FParent.ConvertToLocalTime(CreatedAt);
end;

procedure TXMLStatus.OpenWebTweet;
begin
  OpenWebAsBrowser(GetTweetUrl);
end;

procedure TXMLStatus.SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  dateTime: TDateTime;
  _user: TXMLUser;
  _recipientUser: TXMLUser;
  _retweetedStatus: TXMLStatus;
  s: string;
begin
  _user := TXMLUser.Create(FParent); // ȂƖOƂȂEEE
  _recipientUser := TXMLUser.Create(FParent);

  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];

    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'id') then
        begin
          Self.ID := StrToInt64(grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'text') then
        begin
          s := grandchildXMLNode.Text;
          Self.Text := Utils.HTMLDecode(s);
        end
        else if (childXMLNode.NodeName = 'favorited') then
        begin
          Self.Favorited := StrToBoolDef(grandchildXMLNode.Text, false);
          if (grandchildXMLNode.Text <> 'false') then
          begin
            // s := '' + grandchildXMLNode.Text;
            // OutputDebugStringW(@s[1]);
          end;
        end
        else if (childXMLNode.NodeName = 'source') then
        begin
          FParent.GetSource(Self.Source, grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'in_reply_to_status_id') then
        begin
          Self.InReplyToStatusID := StrToInt64Def(grandchildXMLNode.Text, 0);
        end
        else if (childXMLNode.NodeName = 'created_at') then
        begin
          if (FParent.GetDateTime(grandchildXMLNode.Text, dateTime)) then
          begin
            Self.CreatedAt := dateTime;
          end;
        end
        else if (childXMLNode.NodeName = 'truncated') then
        begin
          Self.Truncated := StrToBoolDef(grandchildXMLNode.Text, false);
        end;
      end
      else if (grandchildXMLNode.NodeType = ntElement) then
      begin
        if (childXMLNode.NodeName = 'retweeted_status') then
        begin
          _retweetedStatus := TXMLStatus.Create(FParent, true);
          _retweetedStatus.SetFromXML(childXMLNode, AAPIType);
          _retweetedStatus := XMLStatusManager.RegisterStatus(_retweetedStatus);
          Self.RetweetedStatusID := _retweetedStatus.ID;
          Self.RetweetedStatus := _retweetedStatus;
        end;
        if (FUserEnabled) then
        begin
          if (childXMLNode.NodeName = 'user') then
          begin
            _user.SetFromXML(grandchildXMLNode);
          end
          else if (childXMLNode.NodeName = 'sender') then
          begin
            _user.SetFromXML(grandchildXMLNode);
          end
          else if (childXMLNode.NodeName = 'recipient') then
          begin
            _recipientUser.SetFromXML(grandchildXMLNode);
          end;
        end;
      end
    end;
  end;

  Self.User := XMLUserManager.RegisterUser(_user);
  // ŐVȂXV
  Self.User.UpdateStatus(Self);

  if (_recipientUser.ID <> 0) then
  begin
    Self.RecipientUser := XMLUserManager.RegisterUser(_recipientUser);
  end
  else
  begin
    _recipientUser.Free;
  end;
end;

{ TXMLUser }

procedure TXMLUser.AdjustColors;
var
  foreBright: Single;
  backBright: Single;
  r, g, b: Integer;
  // backBright2: Single;
  diff: Single;
begin
  foreBright := Utils.GetBrightness(Self.ProfileTextColor);
  backBright := Utils.GetBrightness(Self.ProfileSidebarFillColor);

  diff := Abs(foreBright - backBright);
  if (diff > 0.6) then
  begin
    FProfileTextColor := ProfileTextColor;
    FProfileSidebarFillColor := ProfileSidebarFillColor;
  end
  else
  begin
    if (foreBright > backBright) then
    begin
      if (foreBright < 0.2) then
      begin
        foreBright := backBright + 0.6;
      end
      else if (backBright > 0.8) then
      begin
        backBright := foreBright - 0.6;
      end
      else
      begin
        diff := (0.6 - diff) / 2;
        foreBright := foreBright + diff;
        backBright := backBright - diff;
        if (foreBright < 0.6) then
        begin
          foreBright := 0.6;
        end;
      end;
    end
    else
    begin
      if (backBright < 0.2) then
      begin
        backBright := foreBright + 0.6;
      end
      else if (foreBright > 0.8) then
      begin
        foreBright := backBright - 0.6;
      end
      else
      begin
        diff := (0.6 - diff) / 2;
        foreBright := foreBright - diff;
        backBright := backBright + diff;
        if (backBright < 0.6) then
        begin
          backBright := 0.6;
        end;
      end;
    end;
    FProfileTextColor := Utils.AdjustColor(ProfileTextColor, foreBright);
    FProfileSidebarFillColor := Utils.AdjustColor(ProfileSidebarFillColor, backBright);
  end;
  r := GetRValue(FProfileSidebarFillColor);
  g := GetGValue(FProfileSidebarFillColor);
  b := GetBValue(FProfileSidebarFillColor);
  if (foreBright < 0.5) then
  begin
    r := r + (255 - r) * 2 div 3;
    g := g + (255 - g) * 2 div 3;
    b := b + (255 - b) * 2 div 3;
  end
  else
  begin
    r := r div 3;
    g := g div 3;
    b := b div 3;
  end;
  FProfileSidebarFillColor2 := RGB(r, g, b);
end;

procedure TXMLUser.CopyFrom(AUser: TXMLUser);
begin
  Name := AUser.Name;
  // ScreenName := AUser.ScreenName;
  Location := AUser.Location;
  Description := AUser.Description;
  if (ProfileImageUrl <> AUser.ProfileImageUrl) then
  begin
    ProfileImageUrl := AUser.ProfileImageUrl;
    // ProfileImage.Free;
    // ProfileImage := nil;
    ProfileImageRequested := false;
    ProfileImageUrlChanged := true;
  end;
  Url := AUser.Url;
  IsProtected := AUser.IsProtected;
  FollowersCount := AUser.FollowersCount;
  ProfileBackgroundColor := AUser.ProfileBackgroundColor;
  ProfileBackgroundImageUrl := AUser.ProfileBackgroundImageUrl;
  ProfileBackgroundTile := AUser.ProfileBackgroundTile;
  ProfileTextColor := AUser.ProfileTextColor;
  ProfileLinkColor := AUser.ProfileLinkColor;
  ProfileSidebarFillColor := AUser.ProfileSidebarFillColor;
  ProfileSidebarBorderColor := AUser.ProfileSidebarBorderColor;
  FriendsCount := AUser.FriendsCount;
  FavouritesCount := AUser.FavouritesCount;
  UtcOffset := AUser.UtcOffset;
  TimeZone := AUser.TimeZone;
  Notifications := AUser.Notifications;
  GeoEnabled := AUser.GeoEnabled;
  Verified := AUser.Verified;
  Following := AUser.Following;
  Lang := AUser.Lang;
  StatusesCount := AUser.StatusesCount;
  IsContributorsEnabled := AUser.IsContributorsEnabled;
  if (AUser.Status.Text <> '') then
  begin
    Status.Text := AUser.Status.Text;
  end;
  AdjustColors;
end;

constructor TXMLUser.Create(AParent: TXMLParser);
begin
  FParent := AParent;
  Status := TXMLStatus.Create(AParent, false);
  Lists := TObjectList<TXMLList>.Create;
  ProfileTextColor := clBlack;
  ProfileSidebarFillColor := clWhite;
  AdjustColors;
end;

function TXMLUser.CreateUserRectView(AInsertBreak: boolean; ARetweetStatus: TXMLStatus): TRectView;
var
  RectView: TRectView;
  UrlTextItem: TUrlTextItem;
  plainTextItem: TPlainTextItem;
  UserTextItem: TUserTextItem;
  url_: string;

begin
  RectView := TRectView.Create;
  url_ := Self.Url;
  if (Url = '') then
  begin
    plainTextItem := TPlainTextItem.Create(RectView);
    plainTextItem.FontStyle := plainTextItem.FontStyle + [fsBold];
    RectView.Items.Add(plainTextItem);
    plainTextItem.Text := Self.Name;
  end
  else
  begin
    // [Uz[y[W
    UrlTextItem := TUrlTextItem.Create(RectView);
    UrlTextItem.FontStyle := UrlTextItem.FontStyle + [fsBold];
    UrlTextItem.Text := Self.Name;
    UrlTextItem.Url := Url;
    RectView.Items.Add(UrlTextItem);
  end;

  if (AInsertBreak) then
  begin
    // s
    plainTextItem := TBreakItem.Create(RectView);
  end
  else
  begin
    // Xy[X
    plainTextItem := TPlainTextItem.Create(RectView);
    plainTextItem.Text := '  ';
  end;
  RectView.Items.Add(plainTextItem);

  // [U^CC
  UserTextItem := TUserTextItem.Create(RectView);
  UserTextItem.FontStyle := UserTextItem.FontStyle + [fsBold];
  UserTextItem.Text := '@' + Self.ScreenName;
  UserTextItem.UserName := Self.ScreenName;
  RectView.Items.Add(UserTextItem);

  if Assigned(ARetweetStatus) then
  begin
    if (AInsertBreak) then
    begin
      // s
      plainTextItem := TBreakItem.Create(RectView);
    end
    else
    begin
      // Xy[X
      plainTextItem := TPlainTextItem.Create(RectView);
      plainTextItem.Text := '  ';
    end;
    RectView.Items.Add(plainTextItem);
    plainTextItem := TPlainTextItem.Create(RectView);
    plainTextItem.Text := 'RT by ';
    RectView.Items.Add(plainTextItem);
    // [U^CC
    UserTextItem := TUserTextItem.Create(RectView);
    UserTextItem.Text := '@' + ARetweetStatus.User.ScreenName;
    UserTextItem.UserName := ARetweetStatus.User.ScreenName;
    RectView.Items.Add(UserTextItem);

  end;

  Result := RectView;

end;

destructor TXMLUser.Destroy;
begin
  Lists.Free;
  ProfileImage.Free;
  Status.Free;
  inherited;
end;

function TXMLUser.FollowingIndex: Integer;
begin
  Result := Integer(IsFollowingMe) * 2 + Integer(IsFollowedByMe);
end;

function TXMLUser.GetNameEnclosedInParentheses: string;
var
  RegExp: TPerlRegEx;
begin
  if (Name = '') then
  begin
    Result := ScreenName;
    Exit;
  end;
  Result := Name;
  RegExp := TPerlRegEx.Create(nil);
  try
    RegExp.RegEx := '[\(i](.+)[\)j]';
    RegExp.Subject := Name;
    if (RegExp.Match) then
    begin
      Result := RegExp.SubExpressions[1];
    end;
  finally
    RegExp.Free;
  end;
end;

function TXMLUser.GetNamePronounciation(ALang: string): string;
var
  s: string;
begin
  if (NamePronounciation <> '') then
    Result := NamePronounciation
  else
  begin
    s := GetNameEnclosedInParentheses;
    if Utils.ContainsNonAsciiCharacters(s) or (ALang = 'ja') then
    begin
      Result := s + '';
    end
    else
    begin
      Result := s;
    end;
  end;
end;

function TXMLUser.GetProfileSidebarFillColor: TColor;
begin
  Result := FProfileSidebarFillColor;
end;

function TXMLUser.GetProfileSidebarFillColor2: TColor;
begin
  Result := FProfileSidebarFillColor2;
end;

function TXMLUser.GetProfileTextColor: TColor;
begin
  Result := FProfileTextColor;
end;

function TXMLUser.IsFollowedByMe: boolean;
begin
  Result := XMLUserManager.GetFriends(true, Self.ID);
end;

function TXMLUser.IsFollowingMe: boolean;
begin
  Result := XMLUserManager.GetFriends(false, Self.ID);
end;

function TXMLUser.LocalTimeCreatedAt: TDateTime;
begin
  Result := FParent.ConvertToLocalTime(CreatedAt);
end;

function TXMLUser.LowScreenName: string;
begin
  Result := LowerCase(ScreenName);
end;

procedure TXMLUser.OpenWebHP;
begin
  if (Url <> '') then
  begin
    OpenWebAsBrowser(Url);
  end;
end;

procedure TXMLUser.OpenWebTwitterHome;
var
  _url: string;
begin
  _url := TWITTER_BASE_URL + Self.ScreenName + '/';
  OpenWebAsBrowser(_url);
end;

procedure TXMLUser.SetFromXML(AXMLNode: IXMLNode);
var
  childXMLNode: IXMLNode;
  dateTime: TDateTime;
  i: Integer;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    if (childXMLNode.NodeType = ntText) then
    begin
      if (AXMLNode.NodeName = 'id') then
      begin
        Self.ID := StrToInt64(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'name') then
      begin
        Self.Name := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'screen_name') then
      begin
        Self.ScreenName := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'created_at') then
      begin
        if (FParent.GetDateTime(childXMLNode.Text, dateTime)) then
        begin
          Self.CreatedAt := dateTime;
        end;
      end
      else if (AXMLNode.NodeName = 'location') then
      begin
        Self.Location := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'description') then
      begin
        Self.Description := Utils.HTMLDecode(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'profile_image_url') then
      begin
        Self.ProfileImageUrl := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'url') then
      begin
        Self.Url := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'protected') then
      begin
        Self.IsProtected := StrToBoolDef(childXMLNode.Text, false);
      end
      else if (AXMLNode.NodeName = 'followers_count') then
      begin
        Self.FollowersCount := StrToIntDef(childXMLNode.Text, 0);
      end
      else if (AXMLNode.NodeName = 'profile_background_color') then
      begin
        Self.ProfileBackgroundColor := StrToColor(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'profile_text_color') then
      begin
        Self.ProfileTextColor := StrToColor(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'profile_link_color') then
      begin
        Self.ProfileLinkColor := StrToColor(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'profile_sidebar_fill_color') then
      begin
        Self.ProfileSidebarFillColor := StrToColor(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'profile_sidebar_border_color') then
      begin
        Self.ProfileSidebarBorderColor := StrToColor(childXMLNode.Text);
      end
      else if (AXMLNode.NodeName = 'friends_count') then
      begin
        Self.FriendsCount := StrToIntDef(childXMLNode.Text, 0);
      end
      else if (AXMLNode.NodeName = 'favourites_count') then
      begin
        Self.FavouritesCount := StrToIntDef(childXMLNode.Text, 0);
      end
      else if (AXMLNode.NodeName = 'utc_offset') then
      begin
        Self.UtcOffset := StrToIntDef(childXMLNode.Text, 0);
      end
      else if (AXMLNode.NodeName = 'time_zone') then
      begin
        Self.TimeZone := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'profile_background_image_url') then
      begin
        Self.ProfileBackgroundImageUrl := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'profile_background_tile') then
      begin
        Self.ProfileBackgroundTile := StrToBoolDef(childXMLNode.Text, false);
      end
      else if (AXMLNode.NodeName = 'notifications') then
      begin
        Self.Notifications := StrToBoolDef(childXMLNode.Text, false);
      end
      else if (AXMLNode.NodeName = 'geo_enabled') then
      begin
        Self.GeoEnabled := StrToBoolDef(childXMLNode.Text, false);
      end
      else if (AXMLNode.NodeName = 'following') then
      begin
        Self.Following := StrToBoolDef(childXMLNode.Text, false);
      end
      else if (AXMLNode.NodeName = 'statuses_count') then
      begin
        Self.StatusesCount := StrToIntDef(childXMLNode.Text, 0);
      end
      else if (AXMLNode.NodeName = 'lang') then
      begin
        Self.Lang := childXMLNode.Text;
      end
      else if (AXMLNode.NodeName = 'contributors_enabled') then
      begin
        Self.IsContributorsEnabled := StrToBoolDef(childXMLNode.Text, false);
      end;

    end;
  end;
end;

procedure TXMLUser.UpdateStatus(AStatus: TXMLStatus);
begin
  if (Self.Status.CreatedAt < AStatus.CreatedAt) then
  begin
    Self.Status.CopyFrom(AStatus);
  end;
end;

{ TXMLHash }

procedure TXMLHash.CopyFrom(AHash: TXMLHash);
begin
  Self.HourlyLimit := AHash.HourlyLimit;
  Self.RemainingHits := AHash.RemainingHits;
  Self.ResetTimeInSeconds := AHash.ResetTimeInSeconds;
  Self.ResetTime := AHash.ResetTime;
  Self.LocalResetTime := AHash.LocalResetTime;
end;

constructor TXMLHash.Create(AParent: TXMLParser);
begin
  FParent := AParent;
end;

destructor TXMLHash.Destroy;
begin

  inherited;
end;

procedure TXMLHash.SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  dateTime: TDateTime;
begin

  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'hourly-limit') then
        begin
          Self.HourlyLimit := StrToInt(grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'reset-time-in-seconds') then
        begin
          Self.ResetTimeInSeconds := StrToInt(grandchildXMLNode.Text);
          dateTime := DateUtils.UnixToDateTime(Self.ResetTimeInSeconds);
          LocalResetTime := FParent.ConvertToLocalTime(dateTime);
        end
        else if (childXMLNode.NodeName = 'reset-time') then
        begin
          Self.ResetTime := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'remaining-hits') then
        begin
          Self.RemainingHits := StrToInt(grandchildXMLNode.Text);
        end;
      end;
    end;
  end;

end;

{ TXMLList }

constructor TXMLList.Create(AParent: TXMLParser);
begin
  FParent := AParent;
end;

destructor TXMLList.Destroy;
begin

  inherited;
end;

procedure TXMLList.SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  _user: TXMLUser;
begin
  _user := TXMLUser.Create(FParent);
  // ȂƖOƂȂEEE

  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'id') then
        begin
          Self.ID := StrToInt(grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'name') then
        begin
          Self.Name := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'full_name') then
        begin
          Self.FullName := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'description') then
        begin
          Self.Description := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'subscriber_count') then
        begin
          Self.SubscriberCount := StrToInt(grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'member_count') then
        begin
          Self.MemberCount := StrToInt(grandchildXMLNode.Text);
        end
        else if (childXMLNode.NodeName = 'uri') then
        begin
          Self.Uri := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'mode') then
        begin
          Self.Mode := grandchildXMLNode.Text;
        end;
      end
      else if (grandchildXMLNode.NodeType = ntElement) then
      begin
        if (childXMLNode.NodeName = 'user') then
        begin
          _user.SetFromXML(grandchildXMLNode);
        end
      end;
    end;
  end;

  _user.AdjustColors;
  Self.User := XMLUserManager.RegisterUser(_user);
end;

{ TXMLSearch }

procedure TXMLSearch.CopyFrom(ASearch: TXMLSearch);
begin
  Self.ID := ASearch.ID;
  Self.Name := ASearch.Name;
  Self.Query := ASearch.Query;
  Self.CreatedAt := ASearch.CreatedAt;

end;

constructor TXMLSearch.Create(AParent: TXMLParser);
begin
  FParent := AParent;

end;

destructor TXMLSearch.Destroy;
begin

  inherited;
end;

procedure TXMLSearch.SetFromXML(AXMLNode: IXMLNode; AAPIType: TAPIType);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
  dateTime: TDateTime;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'id') then
        begin
          Self.ID := StrToIntDef(grandchildXMLNode.Text, 0);
        end
        else if (childXMLNode.NodeName = 'name') then
        begin
          Self.Name := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'query') then
        begin
          Self.Query := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'remaining-hits') then
        begin
          if (FParent.GetDateTime(grandchildXMLNode.Text, dateTime)) then
          begin
            Self.CreatedAt := dateTime;
          end;
        end;
      end;
    end;
  end;

end;

end.
