unit tvXMLResponse;

interface

uses
  Generics.Collections, Graphics, ExtCtrls, XMLDoc, XMLIntf, SysUtils;

type

  TXMLResponseData = class
  public
    Url: string;
    Hash: string;
    GlobalHash: string;
    LongUrl: string;
    NewHash: Integer;
    procedure SetFromXML(AXMLNode: IXMLNode);
  end;

  TXMLResponse = class

  public
    StatusCode: Integer;
    StatusText: string;
    ResponseData: TXMLResponseData;
    constructor Create;
    destructor Destroy; override;

    procedure SetFromXML(AXMLNode: IXMLNode);
    procedure Clear;
  end;

implementation

{ TXMLResponse }

procedure TXMLResponse.Clear;
begin
  StatusCode := 0;
  StatusText := '';
  ResponseData.Url := '';
  ResponseData.Hash := '';
  ResponseData.GlobalHash := '';
  ResponseData.LongUrl := '';
end;

constructor TXMLResponse.Create;
begin
  ResponseData := TXMLResponseData.Create;
end;

destructor TXMLResponse.Destroy;
begin
  ResponseData.Free;
  inherited;
end;

procedure TXMLResponse.SetFromXML(AXMLNode: IXMLNode);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'status_code') then
        begin
          Self.StatusCode := StrToIntDef(grandchildXMLNode.Text, 0);
        end
        else if (childXMLNode.NodeName = 'status_txt') then
        begin
          Self.StatusText := grandchildXMLNode.Text;
        end
      end
      else if (childXMLNode.NodeName = 'data') then
      begin
        ResponseData.SetFromXML(childXMLNode);
      end;
    end;
  end;

end;

{ TXMLResponseData }

procedure TXMLResponseData.SetFromXML(AXMLNode: IXMLNode);
var
  childXMLNode: IXMLNode;
  grandchildXMLNode: IXMLNode;
  i, j: Integer;
begin
  for i := 0 to AXMLNode.ChildNodes.Count - 1 do
  begin
    childXMLNode := AXMLNode.ChildNodes[i];
    for j := 0 to childXMLNode.ChildNodes.Count - 1 do
    begin
      grandchildXMLNode := childXMLNode.ChildNodes[j];
      if (grandchildXMLNode.NodeType = ntText) then
      begin
        if (childXMLNode.NodeName = 'url') then
        begin
          Self.Url := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'hash') then
        begin
          Self.Hash := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'global_hash') then
        begin
          Self.GlobalHash := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'long_url') then
        begin
          Self.LongUrl := grandchildXMLNode.Text;
        end
        else if (childXMLNode.NodeName = 'new_hash') then
        begin
          Self.NewHash := StrToIntDef(grandchildXMLNode.Text, 0);
        end;
      end;
    end;
  end;

end;

end.
