unit tvXMLStatusManager;

interface

uses
  Generics.Collections, tvXMLBase, tvDBManager;

type
  TXMLStatusManager = class
  private
    FStatuses: TDictionary<Int64, TXMLStatus>;

  public
    constructor Create;
    destructor Destroy; override;

    function GetStatus(AID: Int64): TXMLStatus;
    function RegisterStatus(AStatus: TXMLStatus): TXMLStatus;

    property Statuses: TDictionary<Int64, TXMLStatus>read FStatuses;
  end;

var
  XMLStatusManager: TXMLStatusManager;

implementation

{ TXMLStatusManager }

constructor TXMLStatusManager.Create;
begin
  FStatuses := TDictionary<Int64, TXMLStatus>.Create;
end;

destructor TXMLStatusManager.Destroy;
var
  Status: TXMLStatus;
begin
  for Status in FStatuses.Values do
  begin
    Status.Free;
  end;
  FStatuses.Free;
  inherited;
end;

function TXMLStatusManager.GetStatus(AID: Int64): TXMLStatus;
var
  Status: TXMLStatus;
begin
  if (FStatuses.TryGetValue(AID, Status)) then
  begin
    Result := Status;
  end
  else
  begin
    Result := nil;
  end;
end;

function TXMLStatusManager.RegisterStatus(AStatus: TXMLStatus): TXMLStatus;
var
  Status: TXMLStatus;
begin
  Status := GetStatus(AStatus.ID);

  if (Status <> nil) then
  begin
    Status.CopyFrom(AStatus);
    AStatus.Free;
    AStatus := Status;
  end
  else
  begin
    FStatuses.Add(AStatus.ID, AStatus);
  end;
  DBManager.InsertOrUpdateStatus(AStatus);
  DBManager.SelectStatusIsRead(AStatus);
  Result := AStatus;
end;

initialization

XMLStatusManager := TXMLStatusManager.Create;

finalization

XMLStatusManager.Free;

end.




