unit utFunctions;

interface

uses
  Windows, StrUtils, Graphics, Forms, Controls, Classes, Character, SysUtils, DateUtils;

type

  Utils = class
  public
    class function CompareString(s1, s2: string): Integer;
    class function HTMLDecode(AStr: string): string;
    class function NotColor(AColor: TColor): TColor;
    class function GetSpecialFolder(Folder: Integer): string;
    class function ColorToInt64(AColor: TColor): Int64;
    class function Int64ToColor(AInt64: Int64): TColor;
    class procedure OutputDebugString(AString: string);
    class procedure EnableControlAll(AControl: TControl; AEnabled: boolean);
    class function ContainsNonAsciiCharacters(s: string): boolean;
    class function GetBrightness(AColor: TColor): Single;
    class function AdjustColor(AColor: TColor; ABrightness: Single): TColor;
    class function AddCommaSeparatedString(var AString: string; AValue: string): string;
    class function GetRelativeDateString(ADateTime: TDateTime; ALang: string): string;
    class procedure ModifyPureBlack(ABitmap: TBitmap);
  end;

implementation

uses
  ShlObj;

{ Utils }

class function Utils.AddCommaSeparatedString(var AString: string; AValue: string): string;
begin
  if (AString <> '') then
  begin
    AString := AString + ',' + AValue;
  end
  else
  begin
    AString := AValue;
  end;
end;

class function Utils.AdjustColor(AColor: TColor; ABrightness: Single): TColor;
var
  br: Single;
  v: Single;
  r, g, b: byte;
begin

  if (ABrightness > 1.0) then
  begin
    ABrightness := 1.0;
  end;

  if (ABrightness < 0.0) then
  begin
    ABrightness := 0.0;
  end;

  br := GetBrightness(AColor);
  if (br < 0.01) then
  begin
    r := Trunc(255.0 * ABrightness);
    g := Trunc(255.0 * ABrightness);
    b := Trunc(255.0 * ABrightness);
  end
  else
  begin
    v := ABrightness / br * GetRValue(AColor);
    if (v > 255.0) then
    begin
      r := 255;
    end
    else
    begin
      r := Trunc(v);
    end;
    v := ABrightness / br * GetGValue(AColor);
    if (v > 255.0) then
    begin
      g := 255;
    end
    else
    begin
      g := Trunc(v);
    end;
    v := ABrightness / br * GetBValue(AColor);
    if (v > 255.0) then
    begin
      b := 255;
    end
    else
    begin
      b := Trunc(v);
    end;
  end;

  Result := RGB(r, g, b);
end;

class function Utils.ColorToInt64(AColor: TColor): Int64;
begin
  Result := AColor;
end;

class function Utils.CompareString(s1, s2: string): Integer;
begin
  if (s1 = s2) then
  begin
    Result := 0;
  end
  else if (s1 > s2) then
  begin
    Result := 1;
  end
  else
  begin
    Result := -1;
  end;
end;

class function Utils.ContainsNonAsciiCharacters(s: string): boolean;
var
  uc: UCS4Char;
  i: Integer;
begin
  Result := false;
  for i := 1 to Length(s) do
  begin
    uc := TCharacter.ConvertToUtf32(s, i);
    if (uc >= 128) then
    begin
      Result := true;
      Exit;
    end;
  end;
end;

class procedure Utils.EnableControlAll(AControl: TControl; AEnabled: boolean);
var
  child: TControl;
  winControl: TWinControl;
  i: Integer;
begin
  AControl.Enabled := AEnabled;

  if (AControl is TWinControl) then
  begin
    winControl := AControl as TWinControl;
    for i := 0 to winControl.ControlCount - 1 do
    begin
      child := winControl.Controls[i];
      Utils.EnableControlAll(child, AEnabled);
    end;

  end;
end;

class function Utils.GetBrightness(AColor: TColor): Single;
var
  r, g, b: Single;
begin
  r := GetRValue(AColor) / 255.0;
  g := GetGValue(AColor) / 255.0;
  b := GetBValue(AColor) / 255.0;
  Result := 0.299 * r + 0.587 * g + 0.114 * b;
end;

class function Utils.GetRelativeDateString(ADateTime: TDateTime; ALang: string): string;
var
  relSec: Int64;
begin
  relSec := DateUtils.SecondsBetween(Now, ADateTime);
  if (relSec < 15) then
  begin
    Result := '10bO';
  end
  else if (relSec < 25) then
  begin
    Result := '20bO';
  end
  else if (relSec < 35) then
  begin
    Result := '30bO';
  end
  else if (relSec < 45) then
  begin
    Result := '40bO';
  end
  else if (relSec < 45 * 60) then
  begin
    Result := Format('%dO', [(relSec + 15) div 60]);
  end
  else if (relSec < 60 * 60 * 24) then
  begin
    Result := Format('%dԑO', [(relSec + 15 * 60) div 3600]);
  end
  else
  begin
    Result := FormatDateTime('yy/mm/dd hh:mm:ss', ADateTime);
  end;
end;

class function Utils.GetSpecialFolder(Folder: Integer): string;
var
  path: array [0 .. MAX_PATH] of Char;
  pidl: PItemIDList;
begin
  SHGetSpecialFolderLocation(Application.Handle, Folder, pidl);
  SHGetPathFromIDList(pidl, path);
  Result := path;
end;

class function Utils.HTMLDecode(AStr: string): string;
var
  s: string;
begin
  s := AStr;
  s := StrUtils.ReplaceStr(s, '&gt;', '>');
  s := StrUtils.ReplaceStr(s, '&lt;', '<');
  s := StrUtils.ReplaceStr(s, '&amp;', '&');
  s := StrUtils.ReplaceStr(s, '&quot;', '"');
  s := StrUtils.ReplaceStr(s, '&apos;', '''');
  Result := s;
end;

class function Utils.Int64ToColor(AInt64: Int64): TColor;
begin
  Result := AInt64;
end;

class procedure Utils.ModifyPureBlack(ABitmap: TBitmap);
var
  BytePtr: PByte;
  BytePtr2: PByte;
  x, y, i: Integer;
  b: byte;
begin
  for y := 0 to ABitmap.Height - 1 do
  begin
    BytePtr := ABitmap.ScanLine[y];
    for x := 0 to ABitmap.Width - 1 do
    begin
      BytePtr2 := BytePtr;
      b := 0;
      for i := 0 to 2 do
      begin
        b := b or BytePtr2^;
        Inc(BytePtr2);
      end;
      if (b = 0) then
      begin
        BytePtr^ := 1;
      end;
      Inc(BytePtr, 4);
    end;
  end;
end;

class function Utils.NotColor(AColor: TColor): TColor;
begin
  Result := RGB($FF - GetRValue(AColor), $FF - GetGValue(AColor), $FF - GetBValue(AColor));
end;

class procedure Utils.OutputDebugString(AString: string);
begin
  OutputDebugStringW(@AString[1]);
end;

end.
