unit utThreadQueue;

interface

uses
  Generics.Collections, SyncObjs, classes;

type
  TQueueItem = class

  end;

  TThreadQueue<T> = class(TQueue<T>)
  private
    FConditionCS: TConditionVariableCS;
    FCriticalSection: TCriticalSection;
  public
    procedure Enqueue(AData: T);
    function Dequeue(): T;

    function GetCount: Integer;
    procedure FreeAllQueues; virtual;
    constructor Create;
    destructor Destroy; override;
  end;

  TThreadObjectQueue<T: TQueueItem> = class(TThreadQueue<T>)
  public
    procedure FreeAllQueues; override;
    destructor Destroy; override;
  end;

implementation

{ TThreadQueue<T> }

constructor TThreadQueue<T>.Create;
begin
  FConditionCS := TConditionVariableCS.Create;
  FCriticalSection := TCriticalSection.Create;
end;

function TThreadQueue<T>.Dequeue;
begin
  FCriticalSection.Enter;
  while (Self.Count = 0) do
  begin
    FConditionCS.WaitFor(FCriticalSection);
  end;
  Result := inherited Dequeue;
  FCriticalSection.Leave;
end;

destructor TThreadQueue<T>.Destroy;
begin
  FCriticalSection.Free;
  FConditionCS.Free;

  inherited;
end;

procedure TThreadQueue<T>.Enqueue(AData: T);
begin
  FCriticalSection.Enter;
  inherited Enqueue(AData);
  FConditionCS.ReleaseAll;
  FCriticalSection.Leave;
end;

procedure TThreadQueue<T>.FreeAllQueues;
begin
end;

function TThreadQueue<T>.GetCount: Integer;
begin
  FCriticalSection.Enter;
  Result := Self.Count;
  FCriticalSection.Leave;
end;

{ TThreadObjectQueue<T> }

destructor TThreadObjectQueue<T>.Destroy;
begin
  FreeAllQueues;
  inherited;
end;

procedure TThreadObjectQueue<T>.FreeAllQueues;
var
  data: T;
begin
  inherited;
  while (Count <> 0) do
  begin
    data := inherited Dequeue;
    data.Free;
  end;
end;

end.
